searchData={"items":[{"type":"behaviour","title":"Ecspanse","doc":"Ecspanse is an Entity Component System (ECS) framework for Elixir.\n\n> ##","ref":"Ecspanse.html"},{"type":"behaviour","title":"note {: .info} - Ecspanse","doc":"> Ecspanse is not a game engine, but a flexible foundation\n> for managing state and building logic, offering features like:\n> - flexible queries with multiple filters\n> - dynamic bidirectional relationships\n> - versatile tagging capabilities\n> - system event subscriptions\n> - asynchronous system execution\n\nThe core structure of the Ecspanse library is:\n\n- `Ecspanse`: The main module used to configure and interact with the library.\n- `Ecspanse.Server`: The server orchestrates the execution of systems and the storage of components, resources, and events.\n- `Ecspanse.Entity`: A simple struct with an ID, serving as a holder for components.\n- `Ecspanse.Component`: A struct that may hold state information or act as a simple label for an entity.\n- `Ecspanse.System`: Holds the application core logic. Systems run every frame, either synchronously or asynchronously.\n- `Ecspanse.Resource`: Global state storage, similar to components but not tied to a specific entity. Resources can only be created, updated, and deleted by synchronously executed systems.\n- `Ecspanse.Query`: A tool for retrieving entities, components, or resources.\n- `Ecspanse.Command`: A mechanism for changing components and resources state. They can only be triggered from a system.\n- `Ecspanse.Event`: A mechanism for triggering events, which can be listened to by systems. It is the way to communicate externally with the systems.","ref":"Ecspanse.html#module-note-info"},{"type":"behaviour","title":"Usage - Ecspanse","doc":"To use Ecspanse, a module needs to be created, invoking `use Ecspanse`. This implements the `Ecspanse` behaviour, so the `setup/1` callback must be defined. All the systems and their execution order are defined in the `setup/1` callback.","ref":"Ecspanse.html#module-usage"},{"type":"behaviour","title":"Examples - Ecspanse","doc":"```elixir\ndefmodule TestServer1 do\n  use Ecspanse, fps_limit: 60\n\n  def setup(data) do\n    data\n    |> add_startup_system(Demo.Systems.SpawnHero)\n    |> add_frame_start_system(Demo.Systems.PurchaseItem)\n    |> add_system(Demo.Systems.MoveHero)\n    |> add_frame_end_system(Ecspanse.System.Timer)\n    |> add_shutdown_system(Demo.Systems.Cleanup)\n  end\nend\n```\n\n> ##","ref":"Ecspanse.html#module-examples"},{"type":"behaviour","title":"Info  {: .info} - Ecspanse","doc":"> The `Ecspanse` system scheduling functions are imported by default\n> for the setup module that `use Ecspanse`","ref":"Ecspanse.html#module-info-info"},{"type":"behaviour","title":"Configuration - Ecspanse","doc":"The following configuration options are available:\n- `:fps_limit` (optional) - the maximum number of frames per second. Defaults to :unlimited.\n\n\n# Special Resources\n\nSome special resources, such as `State` or `FPS`, are created by default by the framework.","ref":"Ecspanse.html#module-configuration"},{"type":"function","title":"Ecspanse.add_frame_end_system/3","doc":"Schedules a frame end system to be executed each frame during the game loop.\n\nA frame end system is executed synchronously at the end of each frame.\nSync systems are executed in the order they were added.","ref":"Ecspanse.html#add_frame_end_system/3"},{"type":"function","title":"Options - Ecspanse.add_frame_end_system/3","doc":"- See the `add_system/3` function for more information about the options.","ref":"Ecspanse.html#add_frame_end_system/3-options"},{"type":"function","title":"Examples - Ecspanse.add_frame_end_system/3","doc":"```elixir\n  Ecspanse.add_frame_end_system(ecspanse_data, Ecspanse.Systems.Timer)\n  ```","ref":"Ecspanse.html#add_frame_end_system/3-examples"},{"type":"function","title":"Ecspanse.add_frame_start_system/3","doc":"Schedules a frame start system to be executed each frame during the game loop.\n\nA frame start system is executed synchronously at the beginning of each frame.\nSync systems are executed in the order they were added.","ref":"Ecspanse.html#add_frame_start_system/3"},{"type":"function","title":"Options - Ecspanse.add_frame_start_system/3","doc":"- See the `add_system/3` function for more information about the options.","ref":"Ecspanse.html#add_frame_start_system/3-options"},{"type":"function","title":"Examples - Ecspanse.add_frame_start_system/3","doc":"```elixir\n  Ecspanse.add_frame_start_system(ecspanse_data, Demo.Systems.PurchaseItem)\n  ```","ref":"Ecspanse.html#add_frame_start_system/3-examples"},{"type":"function","title":"Ecspanse.add_shutdown_system/2","doc":"Schedules a shutdown system.\n\nA shutdown system runs only once when the Ecspanse.Server terminates. Shutdown systems do not take any options.\nThis is useful for cleaning up or saving the game state.","ref":"Ecspanse.html#add_shutdown_system/2"},{"type":"function","title":"Examples - Ecspanse.add_shutdown_system/2","doc":"```elixir\n  Ecspanse.add_shutdown_system(ecspanse_data, Demo.Systems.Cleanup)\n  ```","ref":"Ecspanse.html#add_shutdown_system/2-examples"},{"type":"function","title":"Ecspanse.add_startup_system/2","doc":"Schedules a startup system.\n\nA startup system runs only once during the Ecspanse startup process. Startup systems do not take any options.","ref":"Ecspanse.html#add_startup_system/2"},{"type":"function","title":"Examples - Ecspanse.add_startup_system/2","doc":"```elixir\n  Ecspanse.add_startup_system(ecspanse_data, Demo.Systems.SpawnHero)\n  ```","ref":"Ecspanse.html#add_startup_system/2-examples"},{"type":"function","title":"Ecspanse.add_system/3","doc":"Schedules an async system to be executed each frame during the game loop.","ref":"Ecspanse.html#add_system/3"},{"type":"function","title":"Options - Ecspanse.add_system/3","doc":"- `:run_in_state` - a list of states in which the system should run.\n- `:run_not_in_state` - a list of states in which the system should not run.\n- `:run_if` - a list of tuples containing the module and function that define a condition for running the system. Eg. `[{MyModule, :my_function}]`. The function must return a boolean.\n- `:run_after` - only for async systems - a system or list of systems that must run before this system.","ref":"Ecspanse.html#add_system/3-options"},{"type":"function","title":"Order of execution - Ecspanse.add_system/3","doc":"Systems are executed each frame during the game loop. Sync systems run in the order they were added to the data's operations list.\nAsync systems are grouped in batches depending on the componets they are locking.\nSee the `Ecspanse.System` module for more information about component locking.\n\nThe order in which async systems run can pe specified using the `run_after` option.\nThis option takes a system or list of systems that must be run before the current system.\n\nWhen using the `run_after: SystemModule1` or `run_after: [SystemModule1, SystemModule2]` option, the following rules apply:\n- The system(s) specified in `run_after` must be already scheduled. This prevents circular dependencies.\n- There is a deliberate choice to allow **only the `run_after`** ordering option. While a `run_before` option would simplify some relations, it can also introduce circular dependencies.\n\nExample of circular dependency:\n- System A\n- System B, run_before: System A\n- System C, run_after: System A, run_before: System B\n\n> ##","ref":"Ecspanse.html#add_system/3-order-of-execution"},{"type":"function","title":"Info  {: .info} - Ecspanse.add_system/3","doc":"> The 'run_after' option does not depend on the \"before\" system being executed or not\n> (eg. when the \"before\" system subscribes to an event that is not triggered this frame).\n> The system will be executed anyway, even if the \"before\" system is not executed this frame.\n> The `:run_after` option is evaluated only once, at server start-up,\n> when the async systems are grouped together into batches.\n> Then the scheduler tries to execute every system in every batch.","ref":"Ecspanse.html#add_system/3-info-info"},{"type":"function","title":"Examples - Ecspanse.add_system/3","doc":"```elixir\n  Ecspanse.add_system(\n    ecspanse_data,\n    Demo.Systems.MoveHero,\n    run_in_state: [:play],\n    run_after: [Demo.Systems.RestoreEnergy]\n  )\n  ```","ref":"Ecspanse.html#add_system/3-examples"},{"type":"function","title":"Ecspanse.add_system_set/3","doc":"Convenient way to group together related systems.\n\nNew systems can be added to the set using the `add_system_*` functions.\nSystem sets can also be nested.","ref":"Ecspanse.html#add_system_set/3"},{"type":"function","title":"Options - Ecspanse.add_system_set/3","doc":"The set options that applied on top of each system options in the set.\n- See the `add_system/3` function for more information about the options.","ref":"Ecspanse.html#add_system_set/3-options"},{"type":"function","title":"Examples - Ecspanse.add_system_set/3","doc":"```elixir\ndefmodule Demo do\n  use Ecspanse\n\n  @impl Ecspanse\n  def setup(data) do\n    data\n    |> Ecspanse.add_system_set({Demo.HeroSystemSet, :setup}, [run_in_state: :play])\n  end\n\n  defmodule HeroSystemSet do\n    def setup(data) do\n      data\n      |> Ecspanse.add_system(Demo.Systems.MoveHero, [run_after: Demo.Systems.RestoreEnergy])\n      |> Ecspanse.add_system_set({Demo.ItemsSystemSet, :setup})\n    end\n  end\n\n  defmodule ItemsSystemSet do\n    def setup(data) do\n      data\n      |> Ecspanse.add_system(Demo.Systems.PickupItem)\n    end\n  end\nend\n```","ref":"Ecspanse.html#add_system_set/3-examples"},{"type":"function","title":"Ecspanse.event/2","doc":"Queues an event to be processed in the next frame.","ref":"Ecspanse.html#event/2"},{"type":"function","title":"Options - Ecspanse.event/2","doc":"- `:batch_key` - A key for grouping multiple similar events in different batches within the same frame.\nThe event scheduler groups the events into batches by unique `{EventModule, batch_key}` combinations.\nIn most cases, the key may be an entity ID that either triggers or is impacted by the event.\nDefaults to `default`, meaning that similar events will be placed in separate batches.","ref":"Ecspanse.html#event/2-options"},{"type":"function","title":"Examples - Ecspanse.event/2","doc":"```elixir\n    Ecspanse.event({Demo.Events.MoveHero, direction: :up},  batch_key: hero_entity.id)\n  ```","ref":"Ecspanse.html#event/2-examples"},{"type":"function","title":"Ecspanse.fetch_pid/0","doc":"Retrieves the Ecspanse Server process PID.\nIf the data process is not found, it returns an error.","ref":"Ecspanse.html#fetch_pid/0"},{"type":"function","title":"Examples - Ecspanse.fetch_pid/0","doc":"```elixir\n  Ecspanse.fetch_pid()\n  {:ok, %{name: data_name, pid: data_pid}}\n  ```","ref":"Ecspanse.html#fetch_pid/0-examples"},{"type":"callback","title":"Ecspanse.setup/1","doc":"The `setup/1` callback is called on Ecspanse startup and is the place to define the running systems.\nIt takes an `Ecspanse.Data` struct as an argument and returns an updated struct.","ref":"Ecspanse.html#c:setup/1"},{"type":"callback","title":"Examples - Ecspanse.setup/1","doc":"```elixir\ndefmodule MyProject do\n  use Ecspanse\n\n  @impl Ecspanse\n  def setup(%Ecspanse.Data{} = data) do\n    data\n    |> Ecspanse.Server.add_system(Demo.Systems.MoveHero)\n  end\nend\n```","ref":"Ecspanse.html#c:setup/1-examples"},{"type":"module","title":"Ecspanse.Data","doc":"The `Data` module defines a struct that holds the state of the Ecspanse initialization process.\nThis struct is passed to the `setup/1` callback, which is used to define the running systems.\nAfter the initialization process the `Data` struct is not relevant anymore.","ref":"Ecspanse.Data.html"},{"type":"type","title":"Ecspanse.Data.operation/0","doc":"","ref":"Ecspanse.Data.html#t:operation/0"},{"type":"type","title":"Ecspanse.Data.operations/0","doc":"","ref":"Ecspanse.Data.html#t:operations/0"},{"type":"type","title":"Ecspanse.Data.t/0","doc":"The data used for the Ecspanse initialization process.","ref":"Ecspanse.Data.html#t:t/0"},{"type":"module","title":"Ecspanse.Frame","doc":"The frame is a struct that encapsulates the state of the current frame.\n\nIt holds information such as the time elapsed since the last frame and any batches of events that have been inserted during the previous frame.\nThis frame struct is available to all systems during the frame.","ref":"Ecspanse.Frame.html"},{"type":"module","title":"Fields - Ecspanse.Frame","doc":"- `:event_batches` - a collection of event batches queued for execution within this frame.\n- `:delta` - the time elapsed since the last frame in milliseconds.","ref":"Ecspanse.Frame.html#module-fields"},{"type":"type","title":"Ecspanse.Frame.t/0","doc":"The frame struct.","ref":"Ecspanse.Frame.html#t:t/0"},{"type":"type","title":"Example - Ecspanse.Frame.t/0","doc":"```elixir\n  %Ecspanse.Frame{\n    event_batches: [[%Demo.Events.MoveHero{direction: :left}, %Demo.Events.FindResource{type: :gold}], [%Demo.Events.MoveHero{direction: :down}]],\n    delta: 18,\n  }\n  ```","ref":"Ecspanse.Frame.html#t:t/0-example"},{"type":"module","title":"Ecspanse.Server","doc":"The server is responsible for managing the internal state of the framework,\nscheduling and running the Systems, and batching the Events.\n\nThe server is started by adding the module that invokes `use Ecspanse` to the supervision tree.\n\n> ##","ref":"Ecspanse.Server.html"},{"type":"module","title":"Info  {: .info} - Ecspanse.Server","doc":"> There is only one server instance running per otp app.\n> Trying to create another setup module that `use Ecspanse` and\n> adding it to the supervision tree will result in an error.","ref":"Ecspanse.Server.html#module-info-info"},{"type":"function","title":"Ecspanse.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Ecspanse.Server.html#child_spec/1"},{"type":"function","title":"Ecspanse.Server.debug/0","doc":"Utility function. Returns all the internal state of the framework: `t:Ecspanse.Server.State.t/0`.\n\nThis can be useful for debugging systems scheduling and events batching.\n\n> ##","ref":"Ecspanse.Server.html#debug/0"},{"type":"function","title":"This function is intended for use only in testing and development environments.  {: .warning} - Ecspanse.Server.debug/0","doc":"","ref":"Ecspanse.Server.html#debug/0-this-function-is-intended-for-use-only-in-testing-and-development-environments-warning"},{"type":"function","title":"Ecspanse.Server.test_server/1","doc":"Utility function. The server switches to test mode.\n\nAt the beginning of each frame, a `t:debug_next_frame/0` tupple message will be sent\nto the process passed as function argument.\n\n> ##","ref":"Ecspanse.Server.html#test_server/1"},{"type":"function","title":"This function is intended for use only in testing and development environments.  {: .warning} - Ecspanse.Server.test_server/1","doc":"","ref":"Ecspanse.Server.html#test_server/1-this-function-is-intended-for-use-only-in-testing-and-development-environments-warning"},{"type":"type","title":"Ecspanse.Server.debug_next_frame/0","doc":"Debug server state","ref":"Ecspanse.Server.html#t:debug_next_frame/0"},{"type":"module","title":"Ecspanse.Server.State","doc":"The internal state of the framework.","ref":"Ecspanse.Server.State.html"},{"type":"type","title":"Ecspanse.Server.State.t/0","doc":"The internal state of the framework.","ref":"Ecspanse.Server.State.html#t:t/0"},{"type":"module","title":"Ecspanse.TestServer","doc":"This server is initiated upon application launch when operating in the test environment.\nThis is done to allow tests to start their own custom servers and schedule custom systems.\n\nA basic test system setup may look like this:\n  ```elixir\n  defmodule Demo.Systems.MoveHeroTest do\n    use ExUnit.Case\n\n    defmodule DemoTest do\n      @moduledoc \"A setup that does not schedule any system\"\n      use Ecspanse\n\n      @impl true\n      def setup(data) do\n        data\n      end\n    end\n\n    setup do\n      {:ok, _pid} = start_supervised({DemoTest, :test})\n      Ecspanse.System.debug()\n    end\n  end\n  ```\n\n  In the test `setup` block, the server is started with the tuple `{DemoTest, :test}`.\n  This is needed to point Ecspanse to start the `Ecspanse.Server` with the `DemoTest` setup.\n\n  The `Ecspanse.System.debug/0` call grants the test pid `Ecspanse.System` powers.\n  Meaning that it can runt commands without being in the context of a system.","ref":"Ecspanse.TestServer.html"},{"type":"function","title":"Ecspanse.TestServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Ecspanse.TestServer.html#child_spec/1"},{"type":"module","title":"Ecspanse.Entity","doc":"Entities are only identifiers. An entity exists only if it holds at least one component.\nThe entities per se are not persisted.\n\nEntities are represented as a struct with `id` as the only field.","ref":"Ecspanse.Entity.html"},{"type":"module","title":"Examples - Ecspanse.Entity","doc":"```elixir\n  %Ecspanse.Entity{id: \"cfa1ad89-44b6-4d1f-8590-186354be9158\"}\n  ```","ref":"Ecspanse.Entity.html#module-examples"},{"type":"function","title":"Ecspanse.Entity.fetch/1","doc":"","ref":"Ecspanse.Entity.html#fetch/1"},{"type":"type","title":"Ecspanse.Entity.entity_spec/0","doc":"An `entity_spec` is the definition required to create an entity.","ref":"Ecspanse.Entity.html#t:entity_spec/0"},{"type":"type","title":"Options - Ecspanse.Entity.entity_spec/0","doc":"- `:id` - a custom unique ID for the entity (binary). If not provided, a random UUID will be generated.\n- `:components` - a list of `t:Ecspanse.Component.component_spec/0` to be added to the entity.\n- `:children` A list of `t:Ecspanse.Entity.t/0` to be added as children to the entity. Children entities should already exist.\n- `:parents` A list of `t:Ecspanse.Entity.t/0` to be added as parents to the entity. Parent entities should already exist.\n\n> ##","ref":"Ecspanse.Entity.html#t:entity_spec/0-options"},{"type":"type","title":"Note  {: .info} - Ecspanse.Entity.entity_spec/0","doc":"> At least one of the `:components`, `:children` or `:parents` options must be provided,\n> otherwise the entity cannot be persisted.\n\n> ##","ref":"Ecspanse.Entity.html#t:entity_spec/0-note-info"},{"type":"type","title":"Entity ID  {: .warning} - Ecspanse.Entity.entity_spec/0","doc":"> The entity IDs must be unique. Attention when providing the `:id` option as part of the `entity_spec`.\n> If the provided ID is not unique, spwaning entities will raise an error.","ref":"Ecspanse.Entity.html#t:entity_spec/0-entity-id-warning"},{"type":"type","title":"Ecspanse.Entity.id/0","doc":"","ref":"Ecspanse.Entity.html#t:id/0"},{"type":"type","title":"Ecspanse.Entity.t/0","doc":"The entity struct.","ref":"Ecspanse.Entity.html#t:t/0"},{"type":"behaviour","title":"Ecspanse.Component","doc":"The `Ecspanse.Component` is the basic building block of the ECS architecture, holding the entity state.\nThe components are defined by invoking `use Ecspanse.Component` in their module definition.\n\nAn entity cannot exist without at least a component.\nAnd the other way around, a component cannot exitst without being allocated to an entity.\nThe components hold their own state, and can also be tagged for easy grouping.\n\nThere are two ways of providing the components with their initial state and tags:\n\n1. At compile time, when invoking the `use Ecspanse.Component`, by providing the `:state` and `:tags` options.\n  ```elixir\n  defmodule Demo.Components.Position do\n    use Ecspanse.Component, state: [x: 3, y: 5], tags: [:map]\n  end\n  ```\n\n2. At runtime when creating the components from specs: `t:Ecspanse.Component.component_spec()`:\n  ```elixir\n  Ecspanse.Command.spawn_entity!({Ecspanse.Entity,\n    components: [\n      Hero,\n      {Demo.Components.Position, [x: 7, y: 2], [:map]},\n    ]\n  )\n\n  # or\n\n  Ecspanse.Command.add_component!(hero_entity, {Demo.Components.Position, [x: 7, y: 2], [:map]})\n  ```\n\nAfter their creation, the components become structs with the fields defined in the `state` option of the spec, plus some metadata added by the framework. Components can also be used\nas an Entity lable, without state.\n\nAfter being created, components become structs with the provided fields, along with some metadata added by the framework.\nComponents can also be used as an entity label, without state.\n\nThere are some special components that are created automatically by the framework:\n- `Ecspanse.Component.Children` - holds the list of child entities.\n- `Ecspanse.Component.Parents` - holds the parent entities.","ref":"Ecspanse.Component.html"},{"type":"behaviour","title":"Options - Ecspanse.Component","doc":"- `:state` - a list with all the component state struct keys and their initial values (if any).\nFor example: `[:amount, max_amount: 100]`\n- `:tags` - list of atoms that act as tags for the current component. Defaults to [].\n\n> ##","ref":"Ecspanse.Component.html#module-options"},{"type":"behaviour","title":"Tags  {: .info} - Ecspanse.Component","doc":"> Tags can be added at compile time, and at runtime **only** when creating a new component.\n> They cannot be eddited or removed later on for the existing component.\n>\n> The List of tags added at compile time is merged with the one provided at run time.","ref":"Ecspanse.Component.html#module-tags-info"},{"type":"function","title":"Ecspanse.Component.debug/0","doc":"Utility function. Returns all the components and their state, together with their entity association and tags.\n\n> ##","ref":"Ecspanse.Component.html#debug/0"},{"type":"function","title":"This function is intended for use only in testing and development environments.  {: .warning} - Ecspanse.Component.debug/0","doc":"","ref":"Ecspanse.Component.html#debug/0-this-function-is-intended-for-use-only-in-testing-and-development-environments-warning"},{"type":"callback","title":"Ecspanse.Component.fetch/1","doc":"Fetches the component for an entity. It has the same functionality as `Ecspanse.Query.fetch_component/2`,\nbut it may be more convenient to use in some cases.\n\n> ##","ref":"Ecspanse.Component.html#c:fetch/1"},{"type":"callback","title":"Implemented Callback  {: .tip} - Ecspanse.Component.fetch/1","doc":"> This callback is implemented by the library and can be used as such.","ref":"Ecspanse.Component.html#c:fetch/1-implemented-callback-tip"},{"type":"callback","title":"Examples: - Ecspanse.Component.fetch/1","doc":"``` elixir\n    {:ok, %Demo.Components.Position{} = position_comopnent} = Demo.Components.Position.fetch(hero_entity)\n\n    # it's the same as:\n\n    {:ok, %Demo.Components.Position{} = position_comopnent} = Ecspanse.Query.fetch_component(hero_entity, Demo.Components.Position)\n  ```","ref":"Ecspanse.Component.html#c:fetch/1-examples"},{"type":"callback","title":"Ecspanse.Component.list/0","doc":"Lists all components of the current type for all entities.\n\n> ##","ref":"Ecspanse.Component.html#c:list/0"},{"type":"callback","title":"Implemented Callback  {: .info} - Ecspanse.Component.list/0","doc":"> This callback is implemented by the library and can be used as such.","ref":"Ecspanse.Component.html#c:list/0-implemented-callback-info"},{"type":"callback","title":"Examples: - Ecspanse.Component.list/0","doc":"```elixir\n  enemy_components = Demo.Components.Enemy.list()\n  ```","ref":"Ecspanse.Component.html#c:list/0-examples"},{"type":"callback","title":"Ecspanse.Component.validate/1","doc":"**Optional** callback to validate the component state.\nIt takes the component state struct as the only argument and returns `:ok` or an error tuple.\n\n> ##","ref":"Ecspanse.Component.html#c:validate/1"},{"type":"callback","title":"Info  {: .error} - Ecspanse.Component.validate/1","doc":"> When an error tuple is returned, it raises an exception with the provided error message.\n\n> ##","ref":"Ecspanse.Component.html#c:validate/1-info-error"},{"type":"callback","title":"Note  {: .info} - Ecspanse.Component.validate/1","doc":"> For more complex validations, Ecto schemaless changesets may be useful.\n> - [docs](https://hexdocs.pm/ecto/Ecto.Changeset.html#module-schemaless-changesets)\n> - [article](https://medium.com/very-big-things/towards-maintainable-elixir-the-core-and-the-interface-c267f0da43)","ref":"Ecspanse.Component.html#c:validate/1-note-info"},{"type":"callback","title":"Examples - Ecspanse.Component.validate/1","doc":"```elixir\n  defmodule Demo.Components.Gold do\n    use Ecspanse.Component, state: [amount: 0]\n\n    def validate(%__MODULE__{amount: amount}) do\n      if amount >= 0 do\n        :ok\n      else\n        {:error, \"Gold amount cannot be negative\"}\n      end\n    end\n  end\n  ```","ref":"Ecspanse.Component.html#c:validate/1-examples"},{"type":"type","title":"Ecspanse.Component.component_spec/0","doc":"A `component_spec` is the definition required to create a component.","ref":"Ecspanse.Component.html#t:component_spec/0"},{"type":"type","title":"Examples - Ecspanse.Component.component_spec/0","doc":"```elixir\n  Demo.Components.Gold\n  {Demo.Components.Gold, [amount: 5]}\n  {Demo.Components.Gold, [amount: 5], [:resource, :available]}\n  {Demo.Components.Gold, [], [:resource, :available]}\n  ```","ref":"Ecspanse.Component.html#t:component_spec/0-examples"},{"type":"module","title":"Ecspanse.Component.Children","doc":"The `Children` component is a special component provided by the framework\nto maintain references to an entity's child entities.\n\nDedicated queries and commands are provided to interact with this component.\n\nAn empty `Children` component is automatically added upon entity creation,\neven if no child entities are defined at the time of creation.\n\n> ##","ref":"Ecspanse.Component.Children.html"},{"type":"module","title":"Ecspanse parent-child relationships are **bidirectional associations** {: .info} - Ecspanse.Component.Children","doc":"","ref":"Ecspanse.Component.Children.html#module-ecspanse-parent-child-relationships-are-bidirectional-associations-info"},{"type":"type","title":"Ecspanse.Component.Children.t/0","doc":"Entity's children list.","ref":"Ecspanse.Component.Children.html#t:t/0"},{"type":"module","title":"Ecspanse.Component.Parents","doc":"The `Parents` component is a special component provided by the framework\nto maintain references to an entity's parent entities.\n\nDedicated queries and commands are provided to interact with this component.\n\nAn empty `Parents` component is automatically added upon entity creation,\neven if no parent entities are defined at the time of creation.\n\n> ##","ref":"Ecspanse.Component.Parents.html"},{"type":"module","title":"Ecspanse parent-child relationships are **bidirectional associations** {: .info} - Ecspanse.Component.Parents","doc":"","ref":"Ecspanse.Component.Parents.html#module-ecspanse-parent-child-relationships-are-bidirectional-associations-info"},{"type":"type","title":"Ecspanse.Component.Parents.t/0","doc":"Entity's parents list.","ref":"Ecspanse.Component.Parents.html#t:t/0"},{"type":"module","title":"Ecspanse.System","doc":"The system implements the logic and behaviors of the application\nby manipulating the state of the components.\nThe systems are defined by invoking `use Ecspanse.System` in their module definition.\n\nThe system modules must implement the\n`c:Ecspanse.System.WithoutEventSubscriptions.run/1` or\n`c:Ecspanse.System.WithEventSubscriptions.run/2` callbacks,\ndepending if the system subscribes to certain events or not.\nThe return value of the `run` function is ignored.\n\nThe Ecspanse systems run either synchronously or asynchronously,\nas scheduled in the `c:Ecspanse.setup/1` callback.\n\nSystems are the sole mechanism through which the state of components can be altered.\nRunning commands outside of a system is not allowed.\n\nResources can be created, updated, and deleted only by systems that are executed synchronously.\n\nThere are some special systems that are created automatically by the framework:\n- `Ecspanse.System.CreateDefaultResources` - startup system that creates the default framework resources.\n- `Ecspanse.System.Debug` - used by the `debug/0` function.\n- `Ecspanse.System.Timer` - tracks and updates all components using the `Ecspanse.Template.Component.Timer` template.\n- `Ecspanse.System.TrackFPS` - tracks and updates the `Ecspanse.Resource.FPS` resource.\n\n> ##","ref":"Ecspanse.System.html"},{"type":"module","title":"Info  {: .info} - Ecspanse.System","doc":"> The `Ecspanse.Query` and `Ecspanse.Command` functions are imported by default\n> for all modules that `use Ecspanse.System`","ref":"Ecspanse.System.html#module-info-info"},{"type":"module","title":"Options - Ecspanse.System","doc":"- `:lock_components` - a list of component modules\nthat will be locked for the duration of the system execution.\n- `:event_subscriptions` - a list of event modules that the system subscribes to.","ref":"Ecspanse.System.html#module-options"},{"type":"module","title":"Component locking - Ecspanse.System","doc":"Component locking is required only for async systems to avoid race conditions.\n\nFor async systems, any components that are to be modified, created, or deleted,\nmust be locked in the `lock_components` option. Otherwise, the operation will fail.\nWherever it makes sense, it is recommended to lock also components that are queried but not modified,\nas they could be modified by other systems.\n\nNot all async systems run concurrently. The systems are grouped in batches,\nbased on the components they lock.","ref":"Ecspanse.System.html#module-component-locking"},{"type":"module","title":"Event subscriptions - Ecspanse.System","doc":"The event subscriptions enables a system to execute solely in response to certain specified events.\n\nThe `c:Ecspanse.System.WithEventSubscriptions.run/2` callback is triggered\nfor every occurrence of an event type to which the system has subscribed.\nThese callbacks execute concurrently to enhance performance.\nHowever, they are grouped based on their batch keys (see `Ecspanse.event/2` options)\nas a safeguard against potential race conditions.","ref":"Ecspanse.System.html#module-event-subscriptions"},{"type":"module","title":"Examples - Ecspanse.System","doc":"```elixir\n  defmodule Demo.Systems.Move do\n    @moduledoc \"An async system locking components, that subscribes to an event\"\n    use Ecspanse.System,\n      lock_components: [Demo.Components.Position],\n      event_subscriptions: [Demo.Events.Move]\n\n    def run(%Demo.Events.Move{entity_id: entity_id, direction: direction}, frame) do\n      # move logic\n    end\n  end\n\n  defmodule Demo.Systems.SpawnEnemy do\n    @moduledoc \"A sync system that does not need to lock components, and it is not subscribed to any events\"\n    use Ecspanse.System\n\n    def run(frame) do\n      # spawn logic\n    end\n  end\n  ```","ref":"Ecspanse.System.html#module-examples"},{"type":"function","title":"Ecspanse.System.debug/0","doc":"Utility function. Gives the current process `Ecspanse.System` abilities to execute commands.\n\nThis is a powerful tool for testing and debugging,\nas the promoted process can change the components and resources state\nwithout having to be scheduled like a regular system.\n\nSee `Ecspanse.TestServer` for more details.\n\n> ##","ref":"Ecspanse.System.html#debug/0"},{"type":"function","title":"This function is intended for use only in testing and development environments.  {: .warning} - Ecspanse.System.debug/0","doc":"","ref":"Ecspanse.System.html#debug/0-this-function-is-intended-for-use-only-in-testing-and-development-environments-warning"},{"type":"function","title":"Ecspanse.System.execute_async/3","doc":"Allows running async code inside a system.\n\nBecause commands can run only from inside a system,\nrunning commands in a Task, for example, is not possible.\nThe `execute_async/3` is a wrapper around `Elixir.Task.async_stream/3`\nand is built exactly for this purpose. It allows running commands in parallel.\n\nThe result of the processing is ignored. So the function is suitable for cases\nwhen the result is not important. For example, updating components for a list of entities.\n\n> ##","ref":"Ecspanse.System.html#execute_async/3"},{"type":"function","title":"Info  {: .info} - Ecspanse.System.execute_async/3","doc":"> This function is already imported for all modules that `use Ecspanse.System`","ref":"Ecspanse.System.html#execute_async/3-info-info"},{"type":"function","title":"Options - Ecspanse.System.execute_async/3","doc":"- `:concurrent` - the number of concurrent tasks to run.\nDefaults to the number of schedulers online.\nSee `Elixir.Task.async_stream/5` options for more details.\n\n> ##","ref":"Ecspanse.System.html#execute_async/3-options"},{"type":"function","title":"use with care  {: .error} - Ecspanse.System.execute_async/3","doc":"> While the locked components ensure that no other system is modifying\n> the same components at the same time, the `execute_async/3` does not offer\n> any such guarantees inside the same system.\n>\n> For example, the same component can be modified concurrently, leading to\n> race conditions and inconsistent state.","ref":"Ecspanse.System.html#execute_async/3-use-with-care-error"},{"type":"function","title":"Examples - Ecspanse.System.execute_async/3","doc":"```elixir\n    Ecspanse.System.execute_async(\n      enemy_entities,\n      fn enemy_entity ->\n        # update the enemy components\n      end,\n      concurrent: length(enemy_entities) + 1\n    )\n  ```","ref":"Ecspanse.System.html#execute_async/3-examples"},{"type":"type","title":"Ecspanse.System.system_queue/0","doc":"","ref":"Ecspanse.System.html#t:system_queue/0"},{"type":"type","title":"Ecspanse.System.t/0","doc":"","ref":"Ecspanse.System.html#t:t/0"},{"type":"module","title":"Ecspanse.System.CreateDefaultResources","doc":"Special framework system that creates default resources.\nAutomatically runs only once on startup.","ref":"Ecspanse.System.CreateDefaultResources.html"},{"type":"module","title":"Ecspanse.System.Debug","doc":"Generic system to be used by the `Ecspanse.System.debug/0`\nin dev and test environments.","ref":"Ecspanse.System.Debug.html"},{"type":"module","title":"Ecspanse.System.Timer","doc":"A special system provided by the framework that\ncounts down the time for all the custom timer components.\n\nIf the `timer` functionality is used, this system\nneeds to be manually added in the `c:Ecspanse.setup/1`\ncallback as a sync system.\n\nSee `Ecspanse.Template.Component.Timer` for details.","ref":"Ecspanse.System.Timer.html"},{"type":"module","title":"Ecspanse.System.TrackFPS","doc":"A special system provided by the framework to track the FPS.\n\nThe value is stored in the `Ecspanse.Resource.FPS` resource.","ref":"Ecspanse.System.TrackFPS.html"},{"type":"behaviour","title":"Ecspanse.System.WithEventSubscriptions","doc":"Systems that run only if specific events are triggered.","ref":"Ecspanse.System.WithEventSubscriptions.html"},{"type":"callback","title":"Ecspanse.System.WithEventSubscriptions.run/2","doc":"Runs only if the system is subscribed to the triggering event.\nThe return value is ignored.\n\nIt recives the triggering event struct as the first argument\nand the current `t:Ecspanse.Frame.t/0` struct as the second argument.","ref":"Ecspanse.System.WithEventSubscriptions.html#c:run/2"},{"type":"behaviour","title":"Ecspanse.System.WithoutEventSubscriptions","doc":"Systems that run every frame and do not depend on any event.","ref":"Ecspanse.System.WithoutEventSubscriptions.html"},{"type":"callback","title":"Ecspanse.System.WithoutEventSubscriptions.run/1","doc":"Runs every frame for the current system.\nThe return value is ignored.\n\nIt recives the current `t:Ecspanse.Frame.t/0` struct as the only argument.","ref":"Ecspanse.System.WithoutEventSubscriptions.html#c:run/1"},{"type":"behaviour","title":"Ecspanse.Resource","doc":"Resources are global components that don't belong to any entity.\n\nThey are best used for configuration, global state, statistics, etc.\n\nResources are defined by invoking `use Ecspanse.Resource` in their module definition.","ref":"Ecspanse.Resource.html"},{"type":"behaviour","title":"Options - Ecspanse.Resource","doc":"- `:state` - a list with all the resource state struct keys and their initial values (if any).\nFor example: `[:player_count, max_players: 100]`\n\nThere are two ways of providing the resources with their initial state:\n\n1. At compile time, when invoking the `use Ecspanse.Resource`, by providing the `:state` option.\n  ```elixir\n  defmodule Demo.Resources.PlayerCount do\n    use Ecspanse.Resource, state: [player_count: 0, max_players: 100]\n  end\n  ```\n\n2. At runtime when creating the resources from specs: `t:Ecspanse.Resource.resource_spec()`\n  ```elixir\n  Ecspanse.Command.insert_resource!({Demo.Resources.Lobby, [max_players: 50]})\n  ```\n\nThere are some special resources that are created automatically by the framework:\n- `Ecspanse.Resource.FPS` - tracks the frames per second.\n- `Ecspanse.Resource.State` - a high level state implementation.\n\n> ##","ref":"Ecspanse.Resource.html#module-options"},{"type":"behaviour","title":"Note  {: .info} - Ecspanse.Resource","doc":"> Resources can be created, updated or deleted only from sysnchronous systems.","ref":"Ecspanse.Resource.html#module-note-info"},{"type":"function","title":"Ecspanse.Resource.debug/0","doc":"Utility function. Returns all the resources and their state.\n\n> ##","ref":"Ecspanse.Resource.html#debug/0"},{"type":"function","title":"This function is intended for use only in testing and development environments.  {: .warning} - Ecspanse.Resource.debug/0","doc":"","ref":"Ecspanse.Resource.html#debug/0-this-function-is-intended-for-use-only-in-testing-and-development-environments-warning"},{"type":"callback","title":"Ecspanse.Resource.validate/1","doc":"**Optional** callback to validate the resource state.\n\nSee `c:Ecspanse.Component.validate/1` for more details.","ref":"Ecspanse.Resource.html#c:validate/1"},{"type":"type","title":"Ecspanse.Resource.resource_spec/0","doc":"A `resource_spec` is the definition required to create a resource.","ref":"Ecspanse.Resource.html#t:resource_spec/0"},{"type":"type","title":"Examples - Ecspanse.Resource.resource_spec/0","doc":"```elixir\n  Demo.Resources.Lobby\n  {Demo.Resources.Lobby, [max_players: 50]}\n  ```","ref":"Ecspanse.Resource.html#t:resource_spec/0-examples"},{"type":"module","title":"Ecspanse.Resource.FPS","doc":"A special resource provided by the framework to check the FPS in real-time.\nThe framework also provides a special system that updates the FPS resource.\nThe TrackFPS system needs to be added to the Server, in order to calculate the FPS.\n\n- value: the last second FPS value\n- current: the current frames accumulated this second\n- millisecond: the current millisecond of the second\n\nA special resource provided by the framework for real-time FPS monitoring.\nThe framework also includes a dedicated system that updates this FPS resource.\nTo enable FPS calculation, the `Ecspanse.System.TrackFPS` system must be added\nto the `c:Ecspanse.setup/1` as a sync system.\n\nThe FPS state fields are:\n\n- `:value` - the previous second's FPS value.\n- `:current` - the number of frames within the current second.\n- `:millisecond` - the current millisecond within the second.","ref":"Ecspanse.Resource.FPS.html"},{"type":"function","title":"Ecspanse.Resource.FPS.validate/1","doc":"","ref":"Ecspanse.Resource.FPS.html#validate/1"},{"type":"type","title":"Ecspanse.Resource.FPS.t/0","doc":"","ref":"Ecspanse.Resource.FPS.html#t:t/0"},{"type":"module","title":"Ecspanse.Resource.State","doc":"A special resource provided by the framework to store a generic state.\nThis is a high level state, with an atom type value.\n\nIt is useful in controlling the systems execution. But its use\nis not mandatory.\n\nThe initial state can be set, for example, in a startup system.","ref":"Ecspanse.Resource.State.html"},{"type":"function","title":"Ecspanse.Resource.State.validate/1","doc":"","ref":"Ecspanse.Resource.State.html#validate/1"},{"type":"type","title":"Ecspanse.Resource.State.t/0","doc":"","ref":"Ecspanse.Resource.State.html#t:t/0"},{"type":"module","title":"Ecspanse.Event","doc":"Events act as a one-way communication channel to Ecspanse,\nenabling elements outside of the framework to dispatch data asynchronously into Ecspanse Systems.\nEvents are also used internally to communicate between Systems.\nThe events are defined by invoking `use Ecspanse.Event` in their module definition.\n\nEvents are scheduled using the `Ecspanse.event/2` function.\nAny events scheduled within the current frame will be batched and\nthen made accessible to the systems in the following `t:Ecspanse.Frame.t/0`.\nThe batched events from the current frame are cleared once that frame ends.\nThis implies each system has a single opportunity to process an event that has been scheduled.","ref":"Ecspanse.Event.html"},{"type":"module","title":"Options - Ecspanse.Event","doc":"- `:fields` - a list with all the event struct keys and their initial values (if any)\n  For example: `[:direction, type: :hero]`\n\nAn `inserted_at` field with the `Elixir.System` time of the creation is added to all events automatically.\n\nThere are two ways of providing the events with their field values:\n\n1. At compile time, when invoking the `use Ecspanse.Event`, by providing the `:fields` option.\n  ```elixir\n  defmodule Demo.Events.HeroMoved do\n    use Ecspanse.Event, fields: [:direction, type: :hero]\n  end\n  ```\n\n2. At runtime when creating the events from specs: `t:Ecspanse.Event.event_spec()`\n  ```elixir\n  Ecspanse.event({Demo.Events.HeroMoved, [direction: :left]})\n  ```\n\n> ##","ref":"Ecspanse.Event.html#module-options"},{"type":"module","title":"Note  {: .info} - Ecspanse.Event","doc":"> There are many ways to filter events in the Systems by their struct like:\n>  ```elixir\n>  Enum.filter(events, &match?(%Demo.Events.MoveHero{direction: :right}, &1)) # this allows further pattern matching in the event struct\n>  # or\n>  Enum.filter(events, & &1.__struct__ == Demo.Events.MoveHero)\n>  # or\n>  Enum.filter(events, & fn %event_module{}  -> event_module == Demo.Events.MoveHero end)\n>  ```","ref":"Ecspanse.Event.html#module-note-info"},{"type":"type","title":"Ecspanse.Event.event_spec/0","doc":"An `event_spec` is the definition required to create an event.","ref":"Ecspanse.Event.html#t:event_spec/0"},{"type":"type","title":"Examples - Ecspanse.Event.event_spec/0","doc":"```elixir\n  Demo.Events.MoveHero\n  {Demo.Events.MoveHero, [direction: :left]}\n  ```","ref":"Ecspanse.Event.html#t:event_spec/0-examples"},{"type":"module","title":"Ecspanse.Query","doc":"The `Ecspanse.Query` module provides a set of functions for querying entities, components and resources.\n\nThe queries are read-only operations they do not modify the state of the components or resources.\n\nQueries can be run both from within the Ecspanse systems and from outside of the framework.","ref":"Ecspanse.Query.html"},{"type":"function","title":"Ecspanse.Query.fetch_component/2","doc":"Fetches the component by its module for a given entity.","ref":"Ecspanse.Query.html#fetch_component/2"},{"type":"function","title":"Examples - Ecspanse.Query.fetch_component/2","doc":"```elixir\n  {:ok, gold_component} = Ecspanse.Query.fetch_component(hero_entity, Demo.Components.Gold)\n  ```","ref":"Ecspanse.Query.html#fetch_component/2-examples"},{"type":"function","title":"Ecspanse.Query.fetch_components/2","doc":"Fetches a tuple of components by their modules for a given entity.\nThe entity must have all the components for the query to succeed.","ref":"Ecspanse.Query.html#fetch_components/2"},{"type":"function","title":"Examples - Ecspanse.Query.fetch_components/2","doc":"```elixir\n  {:ok, {gold_component, gems_component}} = Ecspanse.Query.fetch_components(hero_entity, {Demo.Components.Gold, Demo.Components.Gems})\n  ```","ref":"Ecspanse.Query.html#fetch_components/2-examples"},{"type":"function","title":"Ecspanse.Query.fetch_entity/1","doc":"Fetches an `t:Ecspanse.Entity.t/0` by its ID.\n\nAn entity exists only if it has at least one component.","ref":"Ecspanse.Query.html#fetch_entity/1"},{"type":"function","title":"Examples - Ecspanse.Query.fetch_entity/1","doc":"```elixir\n  {:ok, %Ecspanse.Entity{}} = Ecspanse.Query.fetch_entity(hero_entity_id)\n  ```","ref":"Ecspanse.Query.html#fetch_entity/1-examples"},{"type":"function","title":"Ecspanse.Query.fetch_resource/1","doc":"Fetches a resource by its module.","ref":"Ecspanse.Query.html#fetch_resource/1"},{"type":"function","title":"Examples - Ecspanse.Query.fetch_resource/1","doc":"```elixir\n  {:ok, lobby_resource} = Ecspanse.Query.fetch_resource(Demo.Resources.Lobby)\n  ```","ref":"Ecspanse.Query.html#fetch_resource/1-examples"},{"type":"function","title":"Ecspanse.Query.fetch_tagged_component/2","doc":"Fetches an entity's component by a list of tags.\nRaises if more than one entry is found.\n\n> ##","ref":"Ecspanse.Query.html#fetch_tagged_component/2"},{"type":"function","title":"Note  {: .info} - Ecspanse.Query.fetch_tagged_component/2","doc":">\n> The project logic must ensure that only one component per entity is tagged with the given tags.","ref":"Ecspanse.Query.html#fetch_tagged_component/2-note-info"},{"type":"function","title":"Examples - Ecspanse.Query.fetch_tagged_component/2","doc":"```elixir\n  {:ok, %Demo.Components.Paladin{} = hero_class_component} =\n    Ecspanse.Query.fetch_tagged_component(hero_entity, [:class])\n  ```","ref":"Ecspanse.Query.html#fetch_tagged_component/2-examples"},{"type":"function","title":"Ecspanse.Query.get_component_entity/1","doc":"Returns a component's entity.","ref":"Ecspanse.Query.html#get_component_entity/1"},{"type":"function","title":"Examples - Ecspanse.Query.get_component_entity/1","doc":"```elixir\n  {:ok, %Ecspanse.Entity{}} = Ecspanse.Query.get_component_entity(hero_component)\n  ```","ref":"Ecspanse.Query.html#get_component_entity/1-examples"},{"type":"function","title":"Ecspanse.Query.has_children_with_component?/2","doc":"Returns true if the entity has at least a child with the given component module.","ref":"Ecspanse.Query.html#has_children_with_component?/2"},{"type":"function","title":"Examples - Ecspanse.Query.has_children_with_component?/2","doc":"```elixir\n  true = Ecspanse.Query.has_children_with_component?(hero_entity, Demo.Components.Boots)\n  ```","ref":"Ecspanse.Query.html#has_children_with_component?/2-examples"},{"type":"function","title":"Ecspanse.Query.has_children_with_components?/2","doc":"Returns true if the entity has at least a child with all the given component modules.","ref":"Ecspanse.Query.html#has_children_with_components?/2"},{"type":"function","title":"Examples - Ecspanse.Query.has_children_with_components?/2","doc":"```elixir\n  true = Ecspanse.Query.has_children_with_components?(hero_entity, [Demo.Components.Boots, Demo.Components.Sword])\n  ```","ref":"Ecspanse.Query.html#has_children_with_components?/2-examples"},{"type":"function","title":"Ecspanse.Query.has_component?/2","doc":"Returns `true` if the entity has a component with the given module.","ref":"Ecspanse.Query.html#has_component?/2"},{"type":"function","title":"Examples - Ecspanse.Query.has_component?/2","doc":"```elixir\n  true = Ecspanse.Query.has_component?(hero_entity, Demo.Components.Gold)\n  ```","ref":"Ecspanse.Query.html#has_component?/2-examples"},{"type":"function","title":"Ecspanse.Query.has_components?/2","doc":"Returns `true` if the entity has all the components with the given modules.","ref":"Ecspanse.Query.html#has_components?/2"},{"type":"function","title":"Examples - Ecspanse.Query.has_components?/2","doc":"```elixir\n  true = Ecspanse.Query.has_components?(hero_entity, [Demo.Components.Gold, Demo.Components.Gems])\n  ```","ref":"Ecspanse.Query.html#has_components?/2-examples"},{"type":"function","title":"Ecspanse.Query.has_parents_with_component?/2","doc":"Returns true if the entity has at least a parent with the given component module.","ref":"Ecspanse.Query.html#has_parents_with_component?/2"},{"type":"function","title":"Examples - Ecspanse.Query.has_parents_with_component?/2","doc":"```elixir\n  true = Ecspanse.Query.has_parents_with_component?(boots_entity, Demo.Components.Hero)\n  ```","ref":"Ecspanse.Query.html#has_parents_with_component?/2-examples"},{"type":"function","title":"Ecspanse.Query.has_parents_with_components?/2","doc":"Returns true if the entity has at least a parent with all the given component modules.","ref":"Ecspanse.Query.html#has_parents_with_components?/2"},{"type":"function","title":"Examples - Ecspanse.Query.has_parents_with_components?/2","doc":"```elixir\n  true = Ecspanse.Query.has_parents_with_components?(boots_entity, [Demo.Components.Hero, Demo.Components.Gold])\n  ```","ref":"Ecspanse.Query.html#has_parents_with_components?/2-examples"},{"type":"function","title":"Ecspanse.Query.is_child_of?/1","doc":"Reurns `true` if a given entity is a child of another entity.","ref":"Ecspanse.Query.html#is_child_of?/1"},{"type":"function","title":"Examples - Ecspanse.Query.is_child_of?/1","doc":"```elixir\n  true = Ecspanse.Query.is_child_of?(parent: hero_entity, child: boots_entity)\n  ```","ref":"Ecspanse.Query.html#is_child_of?/1-examples"},{"type":"function","title":"Ecspanse.Query.is_parent_of?/1","doc":"Reurns `true` if a given entity is a parent of another entity.","ref":"Ecspanse.Query.html#is_parent_of?/1"},{"type":"function","title":"Examples - Ecspanse.Query.is_parent_of?/1","doc":"```elixir\n  true = Ecspanse.Query.is_parent_of?(parent: hero_entity, child: boots_entity)\n  ```","ref":"Ecspanse.Query.html#is_parent_of?/1-examples"},{"type":"function","title":"Ecspanse.Query.list_ancestors/1","doc":"Returns the list of ancestor entities for the given entity.\nThat means the parents of the entity and their parents and so on.","ref":"Ecspanse.Query.html#list_ancestors/1"},{"type":"function","title":"Examples - Ecspanse.Query.list_ancestors/1","doc":"```elixir\n  [hero_entity, level_entity] = Ecspanse.Query.list_ancestors(compass_entity)\n  ```","ref":"Ecspanse.Query.html#list_ancestors/1-examples"},{"type":"function","title":"Ecspanse.Query.list_children/1","doc":"Returns the list of child entities for the given entity.","ref":"Ecspanse.Query.html#list_children/1"},{"type":"function","title":"Examples - Ecspanse.Query.list_children/1","doc":"```elixir\n  [sword_item_entity, magic_potion_entity] = Ecspanse.Query.list_children(hero_entity)\n  ```","ref":"Ecspanse.Query.html#list_children/1-examples"},{"type":"function","title":"Ecspanse.Query.list_components/1","doc":"Lists all the components for a given entity.\n\nThe output is an unordered list of all the entity's components.\n\n> ##","ref":"Ecspanse.Query.html#list_components/1"},{"type":"function","title":"Note  {: .info} - Ecspanse.Query.list_components/1","doc":">\n> The `Ecspanse.Component.Children` and `Ecspanse.Component.Parents`\n> components are **excluded** from the output.\n>\n> Use the provided `list_children/1` and `list_parents/1` functions to\n> query the entity's relations.","ref":"Ecspanse.Query.html#list_components/1-note-info"},{"type":"function","title":"Examples - Ecspanse.Query.list_components/1","doc":"```elixir\n  [gold_component, gems_component, position_component, energy_component] =\n    Ecspanse.Query.list_components(hero_entity)\n  ```","ref":"Ecspanse.Query.html#list_components/1-examples"},{"type":"function","title":"Ecspanse.Query.list_descendants/1","doc":"Returns the list of descendant entities for the given entity.\nThat means the children of the entity and their children and so on.","ref":"Ecspanse.Query.html#list_descendants/1"},{"type":"function","title":"Examples - Ecspanse.Query.list_descendants/1","doc":"```elixir\n  [inventory_entity, map_entity] = Ecspanse.Query.list_descendants(hero_entity)\n  ```","ref":"Ecspanse.Query.html#list_descendants/1-examples"},{"type":"function","title":"Ecspanse.Query.list_parents/1","doc":"Returns the list of parent entities for the given entity.","ref":"Ecspanse.Query.html#list_parents/1"},{"type":"function","title":"Examples - Ecspanse.Query.list_parents/1","doc":"```elixir\n  [hero_entity] = Ecspanse.Query.list_parents(inventory_entity)\n  ```","ref":"Ecspanse.Query.html#list_parents/1-examples"},{"type":"function","title":"Ecspanse.Query.list_tagged_components/1","doc":"Returns a list of components tagged with a list of tags for all entities.\n\nThe components need to be tagged with all the given tags to return.","ref":"Ecspanse.Query.html#list_tagged_components/1"},{"type":"function","title":"Examples - Ecspanse.Query.list_tagged_components/1","doc":"```elixir\n  [gold_component, gems_component] = Ecspanse.Query.list_tagged_components([:resource, :available])\n  ```","ref":"Ecspanse.Query.html#list_tagged_components/1-examples"},{"type":"function","title":"Ecspanse.Query.list_tagged_components_for_ancestors/2","doc":"Returns a list of components tagged with a list of tags for the ancestors of a given entity.\n\nThe components need to be tagged with all the given tags to return.","ref":"Ecspanse.Query.html#list_tagged_components_for_ancestors/2"},{"type":"function","title":"Examples - Ecspanse.Query.list_tagged_components_for_ancestors/2","doc":"```elixir\n  [dungeon_component] = Ecspanse.Query.list_tagged_components_for_ancestors(hero_entity, [:dungeon])\n  ```","ref":"Ecspanse.Query.html#list_tagged_components_for_ancestors/2-examples"},{"type":"function","title":"Ecspanse.Query.list_tagged_components_for_children/2","doc":"Returns a list of components tagged with a list of tags for the children of a given entity.\n\nThe components need to be tagged with all the given tags to return.","ref":"Ecspanse.Query.html#list_tagged_components_for_children/2"},{"type":"function","title":"Examples - Ecspanse.Query.list_tagged_components_for_children/2","doc":"```elixir\n  [boots_component, compass_component] = Ecspanse.Query.list_tagged_components_for_children(hero_entity, [:inventory])\n  ```","ref":"Ecspanse.Query.html#list_tagged_components_for_children/2-examples"},{"type":"function","title":"Ecspanse.Query.list_tagged_components_for_descendants/2","doc":"Returns a list of components tagged with a list of tags for the descendants of a given entity.\n\nThe components need to be tagged with all the given tags to return.","ref":"Ecspanse.Query.html#list_tagged_components_for_descendants/2"},{"type":"function","title":"Examples - Ecspanse.Query.list_tagged_components_for_descendants/2","doc":"```elixir\n  [orc_component, orc_component, wizard_component] = Ecspanse.Query.list_tagged_components_for_descendants(dungeon_entity, [:enemy])\n  ```","ref":"Ecspanse.Query.html#list_tagged_components_for_descendants/2-examples"},{"type":"function","title":"Ecspanse.Query.list_tagged_components_for_entities/2","doc":"Returns a list of components tagged with a list of tags for a given list of entities.\nThe components are not grouped by entity, but returned as a flat list.\n\nThe components need to be tagged with all the given tags to return.","ref":"Ecspanse.Query.html#list_tagged_components_for_entities/2"},{"type":"function","title":"Examples - Ecspanse.Query.list_tagged_components_for_entities/2","doc":"```elixir\n  [gold_component, gems_component, gems_component] = Ecspanse.Query.list_tagged_components_for_entities([hero_entity, enemy_entity], [:resource, :available])\n  ```","ref":"Ecspanse.Query.html#list_tagged_components_for_entities/2-examples"},{"type":"function","title":"Ecspanse.Query.list_tagged_components_for_entity/2","doc":"Returns a list of components tagged with a list of tags for a given entity.\n\nThe components need to be tagged with all the given tags to return.","ref":"Ecspanse.Query.html#list_tagged_components_for_entity/2"},{"type":"function","title":"Examples - Ecspanse.Query.list_tagged_components_for_entity/2","doc":"```elixir\n  [gold_component, gems_component] = Ecspanse.Query.list_tagged_components_for_entity(hero_entity, [:resource, :available])\n  ```","ref":"Ecspanse.Query.html#list_tagged_components_for_entity/2-examples"},{"type":"function","title":"Ecspanse.Query.list_tagged_components_for_parents/2","doc":"Returns a list of components tagged with a list of tags for the parents of a given entity.\n\nThe components need to be tagged with all the given tags to return.","ref":"Ecspanse.Query.html#list_tagged_components_for_parents/2"},{"type":"function","title":"Examples - Ecspanse.Query.list_tagged_components_for_parents/2","doc":"```elixir\n  [hero_component] = Ecspanse.Query.list_tagged_components_for_parents(boots_entity, [:hero])\n  ```","ref":"Ecspanse.Query.html#list_tagged_components_for_parents/2-examples"},{"type":"function","title":"Ecspanse.Query.list_tags/1","doc":"Lists a component's tags.","ref":"Ecspanse.Query.html#list_tags/1"},{"type":"function","title":"Examples - Ecspanse.Query.list_tags/1","doc":"```elixir\n  [:resource, :available] = Ecspanse.Query.list_tags(gold_component)\n  ```","ref":"Ecspanse.Query.html#list_tags/1-examples"},{"type":"function","title":"Ecspanse.Query.one/1","doc":"Returns a single tuple wiht components for a `t:t/0` query. Returns `nil` if no result was found. Raises if more than one entry.\n\nSee the `select/2` function for more info.","ref":"Ecspanse.Query.html#one/1"},{"type":"function","title":"Ecspanse.Query.select/2","doc":"`select/2` is the most versatile function for querying entities and components.\nOn its own, it will return an `Ecspanse.Query` struct that holds the query details.\nThe struct needs to be passed to `Ecspanse.Query.stream/1` or `Ecspanse.Query.one/1` to get the results.","ref":"Ecspanse.Query.html#select/2"},{"type":"function","title":"Arguments - Ecspanse.Query.select/2","doc":"#","ref":"Ecspanse.Query.html#select/2-arguments"},{"type":"function","title":"1. component_modules - Ecspanse.Query.select/2","doc":"The first argument is a tuple of components to be selected. The query will return the components\nonly for the entities that have **all** the components in the tuple.\n\nThe entity can be queried as well by adding `Ecspanse.Entity` as the first element in the tuple.\nAlso, optional components can be queries, by adding the `:opt` key.\nThe optional components should be placed at the end of the tuple.\n\nThe restults will be returned in the same order as the components in the tuple. This makes it easy to use pattern matching on the result.\n\n#","ref":"Ecspanse.Query.html#select/2-1-component_modules"},{"type":"function","title":"2. filters - Ecspanse.Query.select/2","doc":"The filters are optional. They can be used to further narrow down the results.\n\n- `:with` - a list of components that the entity must have in addition to the ones specified in the `component_modules` tuple.\nBut those components will not be returned in the result. `:with` filter has one option: `:without` - a list of components that the entity must not have.\n- `:or_with` - similar to `:with`. It allows to specify multiple filters for the same query. Multiple `or_with` filters can be used in the same query.\nThe results will be returned if the entity components match any of the filters.\n- `:for` - a list of `t:Ecspanse.Entity.t/0` that the query should be run for. The components will be returned only for those entities.\n- `:not_for` - a list of `t:Ecspanse.Entity.t/0` that the query should not be run for. The components will be returned for all entities except those.\n- `:for_children_of` - a list of `t:Ecspanse.Entity.t/0`. The components will be returned only for the children of those entities.\n- `:for_descendants_of` - a list of `t:Ecspanse.Entity.t/0`. The components will be returned only for all descendants of those entities.\n- `:for_parents_of` - a list of `t:Ecspanse.Entity.t/0`. The components will be returned only for the parents of those entities.\n- `:for_ancestors_of` - a list of `t:Ecspanse.Entity.t/0`. The components will be returned only for all ancestors of those entities.\n\n> ##","ref":"Ecspanse.Query.html#select/2-2-filters"},{"type":"function","title":"Info  {: .error} - Ecspanse.Query.select/2","doc":"> Combining the following filters is not supported: `:for, :not_for, :for_children_of, :for_descendants_of, :for_parents_of, :for_ancestors_of`.\n> Only one of them can be used in a query. Otherwise it will rise an error.","ref":"Ecspanse.Query.html#select/2-info-error"},{"type":"function","title":"Examples - Ecspanse.Query.select/2","doc":"```elixir\n  Ecspanse.Query.select({Ecspanse.Entity, Demo.Components.Health, opt: Demo.Components.Mana},\n    with: [Demo.Components.Orc],\n    or_with: [[Demo.Components.Wizard], without: [Demo.Components.WhiteMagic]],\n    for_descendants_of: [enemy_clan_entity]\n  )\n  |> Ecspanse.Query.stream()\n  |> Enum.to_list()\n  ```\n  a potential result may be:\n  ```elixir\n  [\n    {orc_entity, %Demo.Components.Health{value: 100}, nil},\n    {wizard_entity, %Demo.Components.Health{value: 60}, %Demo.Components.Mana{value: 200}}\n  ]\n  ```","ref":"Ecspanse.Query.html#select/2-examples"},{"type":"function","title":"Ecspanse.Query.stream/1","doc":"Returns a stream of components tuples for a `t:t/0` query.\n\nSee the `select/2` function for more info.","ref":"Ecspanse.Query.html#stream/1"},{"type":"type","title":"Ecspanse.Query.t/0","doc":"The query preparation struct.","ref":"Ecspanse.Query.html#t:t/0"},{"type":"module","title":"Ecspanse.Command","doc":"The `Ecspanse.Command` module provides a set of functions for managing entities, components and resources in the `Ecspanse` engine.\n\nCommands are the only way to change the state of components and resources in `Ecspanse`. These commands can only be run from systems, otherwise an error will be thrown.\nThe `Ecspanse.Command` module includes functions for managing relationships between entities, such as adding and removing children and parents.\nAll entity and component related commands can run for batches (lists) for performance reasons.\n\nAll commands raise an error if the command fails.\n\n##","ref":"Ecspanse.Command.html"},{"type":"module","title":"Entity Relationships - Ecspanse.Command","doc":"The `Ecspanse.Command` module provides functions for managing relationships between entities.\nThis is aslo a powerful tool to manage different kind of collections.\n\n> ##","ref":"Ecspanse.Command.html#module-entity-relationships"},{"type":"module","title":"Ecspanse entities relationships are **bidirectional associations** {: .info} - Ecspanse.Command","doc":"> When adding or removing children or parents, they are automatically added or removed from the corresponding parent or children entities.\n> The same applies when despawning entities.","ref":"Ecspanse.Command.html#module-ecspanse-entities-relationships-are-bidirectional-associations-info"},{"type":"function","title":"Ecspanse.Command.add_child!/2","doc":"Adds an entity as child to a parent entity.","ref":"Ecspanse.Command.html#add_child!/2"},{"type":"function","title":"Examples - Ecspanse.Command.add_child!/2","doc":"```elixir\n  :ok = Ecspanse.Command.add_child!(hero_entity, sword_entity)\n  ```","ref":"Ecspanse.Command.html#add_child!/2-examples"},{"type":"function","title":"Ecspanse.Command.add_children!/1","doc":"The same as `add_child!/2` but can perform multiple operations at once.\nFor example, adding multiple children to multiple parents.\n\nIt takes a list of two element tuples as argument, where the first element of the tuple is the parent entity\nand the second element is a list of children entities.","ref":"Ecspanse.Command.html#add_children!/1"},{"type":"function","title":"Examples - Ecspanse.Command.add_children!/1","doc":"```elixir\n  :ok = Ecspanse.Command.add_children!([\n    {hero_entity, [sword_entity]},\n    {market_entity, [map_entity, potion_entity]}\n  ])\n  ```","ref":"Ecspanse.Command.html#add_children!/1-examples"},{"type":"function","title":"Ecspanse.Command.add_component!/2","doc":"Adds a new component to the specified entity.\n\n> ##","ref":"Ecspanse.Command.html#add_component!/2"},{"type":"function","title":"Info  {: .info} - Ecspanse.Command.add_component!/2","doc":"> An entity cannot have multiple components of the same type.\n> If an attempt is made to insert a component that already exists for the entity, an error will be raised.","ref":"Ecspanse.Command.html#add_component!/2-info-info"},{"type":"function","title":"Examples - Ecspanse.Command.add_component!/2","doc":"```elixir\n  :ok = Ecspanse.Command.add_component!(hero_entity, Demo.Components.Gold)\n  :ok = Ecspanse.Command.add_component!(hero_entity, {Demo.Components.Gold, [amount: 5], [:resource, :available]})\n  ```","ref":"Ecspanse.Command.html#add_component!/2-examples"},{"type":"function","title":"Ecspanse.Command.add_components!/1","doc":"The same as `add_component!/2` but adds multiple components to multiple entities at once.\n\nIt takes a list of two element tuples as argument, where the first element of the tuple is the entity\nand the second element is a list of component specs.","ref":"Ecspanse.Command.html#add_components!/1"},{"type":"function","title":"Examples - Ecspanse.Command.add_components!/1","doc":"```elixir\n  :ok = Ecspanse.Command.add_components!([\n    {inventory_item_entity, [Demo.Components.Sword]},\n    {hero_entity, [Demo.Components.Position, Demo.Components.Hero]}\n  ])\n  ```","ref":"Ecspanse.Command.html#add_components!/1-examples"},{"type":"function","title":"Ecspanse.Command.add_parent!/2","doc":"Adds a parent entity to a child entity.","ref":"Ecspanse.Command.html#add_parent!/2"},{"type":"function","title":"Examples - Ecspanse.Command.add_parent!/2","doc":"```elixir\n  :ok = Ecspanse.Command.add_parent!(sowrd_entity, hero_entity)\n  ```","ref":"Ecspanse.Command.html#add_parent!/2-examples"},{"type":"function","title":"Ecspanse.Command.add_parents!/1","doc":"The same as `add_parent!/2` but can perform multiple operations at once.\nFor example, adding multiple parents to multiple children.\n\nIt takes a list of two element tuples as argument, where the first element of the tuple is the child entity\nand the second element is a list of parent entities.","ref":"Ecspanse.Command.html#add_parents!/1"},{"type":"function","title":"Examples - Ecspanse.Command.add_parents!/1","doc":"```elixir\n  :ok = Ecspanse.Command.add_parents!([\n    {sword_entity, [hero_entity]},\n    {map_entity, [market_entity, vendor_entity]}\n  ])\n  ```","ref":"Ecspanse.Command.html#add_parents!/1-examples"},{"type":"function","title":"Ecspanse.Command.clone_entity!/2","doc":"Clones the specified entity and returns a new entity with the same components.\n\nDue to the potentially large number of components that may be affected by this operation,\nit is recommended to run this function in a synchronous system (such as a `frame_start` or `frame_end` system)\nto avoid the need to lock all involved components.\n\n> ##","ref":"Ecspanse.Command.html#clone_entity!/2"},{"type":"function","title":"Note  {: .info} - Ecspanse.Command.clone_entity!/2","doc":"> The entity's `Ecspanse.Component.Children` and `Ecspanse.Component.Parents` components are not cloned.\n> Use `deep_clone_entity!/2` to clone the entity and all of its descendants.","ref":"Ecspanse.Command.html#clone_entity!/2-note-info"},{"type":"function","title":"Options - Ecspanse.Command.clone_entity!/2","doc":"- `:id` - a custom unique ID for the entity (binary). If not provided, a random UUID will be generated.\n\n> ##","ref":"Ecspanse.Command.html#clone_entity!/2-options"},{"type":"function","title":"Entity ID  {: .warning} - Ecspanse.Command.clone_entity!/2","doc":"> The entity IDs must be unique. Attention when providing the `:id` option.\n> If the provided ID is not unique, clonning the entity will raise an error.","ref":"Ecspanse.Command.html#clone_entity!/2-entity-id-warning"},{"type":"function","title":"Examples - Ecspanse.Command.clone_entity!/2","doc":"```elixir\n  %Ecspanse.Entity{} = entity = Ecspanse.Command.clone_entity!(compass_entity)\n  ```","ref":"Ecspanse.Command.html#clone_entity!/2-examples"},{"type":"function","title":"Ecspanse.Command.deep_clone_entity!/2","doc":"Clones the specified entity and all of its descendants and returns the newly cloned entity.\n\nDue to the potentially large number of components that may be affected by this operation,\nit is recommended to run this function in a synchronous system (such as a `frame_start` or `frame_end` system)\nto avoid the need to lock all involved components.","ref":"Ecspanse.Command.html#deep_clone_entity!/2"},{"type":"function","title":"Options - Ecspanse.Command.deep_clone_entity!/2","doc":"- `:id` - a custom unique ID for the entity (binary). If not provided, a random UUID will be generated.\n\n> ##","ref":"Ecspanse.Command.html#deep_clone_entity!/2-options"},{"type":"function","title":"Entity ID  {: .warning} - Ecspanse.Command.deep_clone_entity!/2","doc":"> The entity IDs must be unique. Attention when providing the `:id` option.\n> If the provided ID is not unique, clonning the entity will raise an error.\n\nThe cloned descendants entities will receive a random UUID as ID by default.","ref":"Ecspanse.Command.html#deep_clone_entity!/2-entity-id-warning"},{"type":"function","title":"Cloning descendants - Ecspanse.Command.deep_clone_entity!/2","doc":"The deep clonning operates only for the descendants of the entity.\nIf any of the descendants has a parent that is not a descendant of the entity,\nthe parent will not be cloned or referenced.\n\nIf this is a desired behaviour, the parents should be added manually after the deep clonning.","ref":"Ecspanse.Command.html#deep_clone_entity!/2-cloning-descendants"},{"type":"function","title":"Examples - Ecspanse.Command.deep_clone_entity!/2","doc":"```elixir\n  %Ecspanse.Entity{} = entity = Ecspanse.Command.deep_clone_entity!(enemy_entity)\n  ```","ref":"Ecspanse.Command.html#deep_clone_entity!/2-examples"},{"type":"function","title":"Ecspanse.Command.delete_resource!/1","doc":"Deletes an existing global resource.","ref":"Ecspanse.Command.html#delete_resource!/1"},{"type":"function","title":"Examples - Ecspanse.Command.delete_resource!/1","doc":"```elixir\n  :ok = Ecspanse.Command.delete_resource!(lobby_resource)\n  ```","ref":"Ecspanse.Command.html#delete_resource!/1-examples"},{"type":"function","title":"Ecspanse.Command.despawn_entities!/1","doc":"The same as `despawn_entity!/1` but despawns multiple entities at once.\nIt takes a list of entities as argument and returns `:ok`.  See `despawn_entity!/1` for more details.","ref":"Ecspanse.Command.html#despawn_entities!/1"},{"type":"function","title":"Ecspanse.Command.despawn_entities_and_descendants!/1","doc":"The same as `despawn_entity_and_descendants!/1` but despawns multiple entities and their descendants at once.\nIt takes a list of entities as argument and returns `:ok`.","ref":"Ecspanse.Command.html#despawn_entities_and_descendants!/1"},{"type":"function","title":"Ecspanse.Command.despawn_entity!/1","doc":"Despawns the specified entity and removes all of its components.\nIt also removes the despawned entity from its parent and child entities, if any.\n\nDue to the potentially large number of components that may be affected by this operation,\nit is recommended to run this function in a synchronous system (such as a `frame_start` or `frame_end` system)\nto avoid the need to lock all involved components.","ref":"Ecspanse.Command.html#despawn_entity!/1"},{"type":"function","title":"Examples - Ecspanse.Command.despawn_entity!/1","doc":"```elixir\n  :ok = Ecspanse.Command.despawn_entity!(hero_entity)\n  ```","ref":"Ecspanse.Command.html#despawn_entity!/1-examples"},{"type":"function","title":"Ecspanse.Command.despawn_entity_and_descendants!/1","doc":"The same as `despawn_entity!/1` but recursively despawns also all descendant tree of the entity.\n\nThis means that it will despawn the children of the entity, and their children, and so on.\n\nIt is an efficient way to remove an entire entity tree with just one operation.\nExtra attention required for entities with shared children.\n\nSee `despawn_entity!/1` for more details.","ref":"Ecspanse.Command.html#despawn_entity_and_descendants!/1"},{"type":"function","title":"Ecspanse.Command.insert_resource!/1","doc":"Inserts a new global resource.\n\n> ##","ref":"Ecspanse.Command.html#insert_resource!/1"},{"type":"function","title":"Info  {: .info} - Ecspanse.Command.insert_resource!/1","doc":"> An Ecspanse instance can only hold one resource of each type at a time.\n> If an attempt is made to insert a resource that already exists, an error will be raised.","ref":"Ecspanse.Command.html#insert_resource!/1-info-info"},{"type":"function","title":"Examples - Ecspanse.Command.insert_resource!/1","doc":"```elixir\n  :ok = Ecspanse.Command.insert_resource!({Demo.Resources.Lobby, player_count: 0})\n  ```","ref":"Ecspanse.Command.html#insert_resource!/1-examples"},{"type":"function","title":"Ecspanse.Command.remove_child!/2","doc":"Removes a child entity from a parent entity.","ref":"Ecspanse.Command.html#remove_child!/2"},{"type":"function","title":"Examples - Ecspanse.Command.remove_child!/2","doc":"```elixir\n  :ok = Ecspanse.Command.remove_child!(hero_entity, sword_entity)\n  ```","ref":"Ecspanse.Command.html#remove_child!/2-examples"},{"type":"function","title":"Ecspanse.Command.remove_children!/1","doc":"The same as `remove_child!/2` but can perform multiple operations at once.\nFor example, removing multiple children from multiple parents.\n\nIt takes a list of two element tuples as argument, where the first element of the tuple is the parent entity\nand the second element is a list of children entities.","ref":"Ecspanse.Command.html#remove_children!/1"},{"type":"function","title":"Examples - Ecspanse.Command.remove_children!/1","doc":"```elixir\n  :ok = Ecspanse.Command.remove_children!([\n    {hero_entity, [sword_entity]},\n    {market_entity, [map_entity, potion_entity]}\n  ])\n  ```","ref":"Ecspanse.Command.html#remove_children!/1-examples"},{"type":"function","title":"Ecspanse.Command.remove_component!/1","doc":"Removes an existing component from its entity. The components is destroyed.","ref":"Ecspanse.Command.html#remove_component!/1"},{"type":"function","title":"Examples - Ecspanse.Command.remove_component!/1","doc":"```elixir\n  :ok = Ecspanse.Command.remove_component!(invisibility_component)\n  ```","ref":"Ecspanse.Command.html#remove_component!/1-examples"},{"type":"function","title":"Ecspanse.Command.remove_components!/1","doc":"The same as `remove_component!/1` but removes multiple components at once.","ref":"Ecspanse.Command.html#remove_components!/1"},{"type":"function","title":"Ecspanse.Command.remove_parent!/2","doc":"Removes a parent entity from a child entity.","ref":"Ecspanse.Command.html#remove_parent!/2"},{"type":"function","title":"Examples - Ecspanse.Command.remove_parent!/2","doc":"```elixir\n  :ok = Ecspanse.Command.remove_parent!(sword_entity, hero_entity)\n  ```","ref":"Ecspanse.Command.html#remove_parent!/2-examples"},{"type":"function","title":"Ecspanse.Command.remove_parents!/1","doc":"The same as `remove_parent!/2` but can perform multiple operations at once.\nFor example, removing multiple parents from multiple children.\n\nIt takes a list of two element tuples as argument, where the first element of the tuple is the child entity\nand the second element is a list of parent entities.","ref":"Ecspanse.Command.html#remove_parents!/1"},{"type":"function","title":"Examples - Ecspanse.Command.remove_parents!/1","doc":"```elixir\n  :ok = Ecspanse.Command.remove_parents!([\n    {sword_entity, [hero_entity]},\n    {map_entity, [market_entity, vendor_entity]}\n  ])\n  ```","ref":"Ecspanse.Command.html#remove_parents!/1-examples"},{"type":"function","title":"Ecspanse.Command.spawn_entities!/1","doc":"The same as `spawn_entity!/1` but spawns multiple entities at once.\nIt takes a list of entity specs as argument and returns a list of Ecspanse.Entity structs.\n\nSee `spawn_entity!/1` for more details.","ref":"Ecspanse.Command.html#spawn_entities!/1"},{"type":"function","title":"Ecspanse.Command.spawn_entity!/1","doc":"Spawns a new entity with the given components and relations provided by the Ecspanse.Entity.entity_spec() type.\nWhen creating a new entity, at least one of the `components:`, `children:` or `parents:`\nmust be provided in the entity spec, otherwise the entity cannot be persisted.\n\nDue to the potentially large number of components that may be affected by this operation,\nit is recommended to run this function in a synchronous system (such as a `frame_start` or `frame_end` system)\nto avoid the need to lock all involved components.","ref":"Ecspanse.Command.html#spawn_entity!/1"},{"type":"function","title":"Examples - Ecspanse.Command.spawn_entity!/1","doc":"```elixir\n    %Ecspanse.Entity{} = Ecspanse.Command.spawn_entity!(\n      {\n        Ecspanse.Entity,\n        id: \"my_custom_id\",\n        components: [Demo.Components.Hero, {Demo.Components.Position, [x: 5, y: 3], [:hero, :map]}],\n        children: [potion_entity, sword_entity],\n        parents: [map_entity]\n      }\n    )\n  ```","ref":"Ecspanse.Command.html#spawn_entity!/1-examples"},{"type":"function","title":"Ecspanse.Command.update_component!/2","doc":"Updates the state of an existing component.\n\nThe function takes two arguments: the component struct to update and a keyword list of changes to apply.","ref":"Ecspanse.Command.html#update_component!/2"},{"type":"function","title":"Examples - Ecspanse.Command.update_component!/2","doc":"```elixir\n  :ok = Ecspanse.Command.update_component!(position_component, x: :12)\n  ```","ref":"Ecspanse.Command.html#update_component!/2-examples"},{"type":"function","title":"Ecspanse.Command.update_components!/1","doc":"The same as `update_component!/2` but updates multiple components at once.\n\nIt takes a list of two element tuples as argument, where the first element of the tuple is the component struct\nand the second element is a keyword list of changes to apply.","ref":"Ecspanse.Command.html#update_components!/1"},{"type":"function","title":"Examples - Ecspanse.Command.update_components!/1","doc":"```elixir\n  :ok = Ecspanse.Command.update_components!([\n    {position_component, x: 7, y: 9},\n    {gold_component, amount: 12}\n  ])\n  ```","ref":"Ecspanse.Command.html#update_components!/1-examples"},{"type":"function","title":"Ecspanse.Command.update_resource!/2","doc":"Updates an existing global resource.","ref":"Ecspanse.Command.html#update_resource!/2"},{"type":"function","title":"Examples - Ecspanse.Command.update_resource!/2","doc":"```elixir\n  :ok = Ecspanse.Command.update_resource!(lobby_resource, player_count: 1)\n  ```","ref":"Ecspanse.Command.html#update_resource!/2-examples"},{"type":"behaviour","title":"Ecspanse.Template.Component","doc":"A template component is a generic way of defining the structure for related components.\nThey share the same state fields and tags.\n\nThe template component, coupled with tags allow a flexible way to manage collections of related components.\nSee `Ecspanse.Component` for more details.\n\nThe template component guarantees that the component that uses it will have certain fields in its state and certain tags.\nThe component itself can define additional fields or tags, specific to its implementation.\nIt can also override the initial values of the template fields.\n\nThe framework embeds some predefined component templates:\n- `Ecspanse.Template.Component.Timer` - a component template that is used to create timer components.","ref":"Ecspanse.Template.Component.html"},{"type":"behaviour","title":"Options - Ecspanse.Template.Component","doc":"See `Ecspanse.Component` for the list of options.","ref":"Ecspanse.Template.Component.html#module-options"},{"type":"behaviour","title":"Examples - Ecspanse.Template.Component","doc":"```elixir\n    defmodule Demo.Componenets.Resource do\n      use Ecspanse.Template.Component, state: [amount: 0], tags: [:resource]\n    end\n\n    defmodule Demo.Componenets.Gold do\n      use Demo.Components.Resource, state: [amount: 5, exchange_rate: 2], tags: [:available]\n    end\n  ```","ref":"Ecspanse.Template.Component.html#module-examples"},{"type":"callback","title":"Ecspanse.Template.Component.validate/1","doc":"**Optional** callback to validate the template and component state fields.\n\nIt runs only at compile time, and it takes the list of fields as the only argument and returns `:ok` or an error tuple.\n\n\n> ##","ref":"Ecspanse.Template.Component.html#c:validate/1"},{"type":"callback","title":"Info  {: .error} - Ecspanse.Template.Component.validate/1","doc":"> When an error tuple is returned, it raises an exception with the provided error message.\n\nFor runtime component state validation see `c:Ecspanse.Component.validate/1`.","ref":"Ecspanse.Template.Component.html#c:validate/1-info-error"},{"type":"callback","title":"Examples - Ecspanse.Template.Component.validate/1","doc":"```elixir\n  defmodule Demo.Components.Resources do\n    use Ecspanse.Template.Component, state: [amount: 0]\n\n    def validate(state_fields) do\n      amount = Keyword.get(state_fields, :amount, 0)\n      if is_integer(amount) and amount >= 0 do\n        :ok\n      else\n        {:error, \"Invalid amount value\"}\n      end\n    end\n  end\n  ```","ref":"Ecspanse.Template.Component.html#c:validate/1-examples"},{"type":"module","title":"Ecspanse.Template.Component.Timer","doc":"The `Timer` is a **Template Component** designed to facilitate the creation\nof custom timer (countdown) components.\n\nIt serves as a foundation for building custom timer components with `use Ecspanse.Template.Component.Timer`.\n\nThe framework automatically decrements the Timer's time each frame,\neliminating the need for manual updates.\nHowever, manual resetting may be necessary under certain circumstances such as:\n- When game logic requires custom resetting.\n- When the timer mode is set to `:once`, necessitating manual reset after reaching 0.\n\nA dedicated `Ecspanse.System.Timer` system is provided by the framework.\nThis system auto-decrements the Timer component's time\nand dispatches an event when time reaches 0.\nTo ensure functionality, this System must be manually included in the `c:Ecspanse.setup/1`.\nNote that it should be added as a sync system, either at frame start or end.\nThis design choice allows developers control over timer operation\nbased on specific states or conditions.\nFor instance, pausing the timers when the game is in a pause state or other game states.\n\nPause control at a granular level can be achieved by setting the `paused` field to true.\n\nThe Timer component template comes with a **predefined state** comprising of:\n- `:duration` - the timer's duration in milliseconds which also serves\nas the reset value.\n- `:time` - the current time of the timer in milliseconds,\nauto-decremented by the framework each frame.\n- `:event` - the event module dispatched when timer reaches 0.\n  - create special timer events using `Ecspanse.Template.Event.Timer`.\n  - these events require no options.\n  - their state is predefined to `%CustomEventModule{entity_id: entity_id}`,\n  where entity refers to owner of the custom timer component.\n  - event batch key corresponds to the component's owner entity's id.\n- `:mode` - defines how timer operates and can be one of:\n  - `:repeat` (default) - timer resets to original duration\n  after reaching 0 and repeats indefinitely.\n  - `:once` - timer runs once and pauses after reaching 0.\n  Time value needs manual reset.\n  - `:temporary`: Timer runs once and removes itself from entity after reaching 0.\n- `paused`: A boolean indicating if timer is paused (defaults to `false`).","ref":"Ecspanse.Template.Component.Timer.html"},{"type":"module","title":"Example: - Ecspanse.Template.Component.Timer","doc":"```elixir\n  defmodule Demo.Components.RestoreEnergyTimer do\n    use Ecspanse.Template.Component.Timer,\n      state: [duration: 3000, time: 3000, event: Demo.Events.RestoreEnergy, mode: :repeat, paused: false]\n  end\n\n  defmodule Demo.Events.RestoreEnergy do\n    use Ecspanse.Template.Template.Event.Timer\n  end\nend\n```\n\nSee [a working example](./tutorial.md#energy-regeneration) in the tutorial","ref":"Ecspanse.Template.Component.Timer.html#module-example"},{"type":"module","title":"Ecspanse.Template.Event","doc":"A template event is a generic way of defining the structure for related events that share the same fields.\n\nThe template event guarantees that the event that uses it will have certain fields.\nThe event itself can define additional fields, specific to its implementation.\n\nThe framework embeds some predefined event templates:\n- `Ecspanse.Template.Event.Timer` - an event template that is used to create timer events","ref":"Ecspanse.Template.Event.html"},{"type":"module","title":"Options - Ecspanse.Template.Event","doc":"See `Ecspanse.Event` for the list of options.","ref":"Ecspanse.Template.Event.html#module-options"},{"type":"module","title":"Examples - Ecspanse.Template.Event","doc":"```elixir\n    defmodule Demo.Events.ConsumerResource do\n      use Ecspanse.Template.Event, fields: [:amount]\n    end\n\n    defmodule Demo.Events.ConsumerGold do\n      use Demo.Events.ConsumeResource, fields: [:amount, :hero_entity_id]\n    end\n  ```","ref":"Ecspanse.Template.Event.html#module-examples"},{"type":"module","title":"Ecspanse.Template.Event.Timer","doc":"The `Timer` is a **Template Event** designed to facilitate the creation\nof custom timer (countdown) events.\n\nIt serves as a foundation for building custom timer events with `use Ecspanse.Template.Event.Timer`.\nIt takes no options.\n\nThe event that will be dispatched by `Event.System.Timer` system when\nthe timer component reaches 0.\n\nTheir state is predefined to `%CustomEventModule{entity_id: entity_id}`,\nwhere entity refers to owner of the custom timer component.","ref":"Ecspanse.Template.Event.Timer.html"},{"type":"module","title":"Example: - Ecspanse.Template.Event.Timer","doc":"```elixir\n  defmodule EnergyRestoreTimer do\n    use Ecspanse.Template.Event.Timer\n  end\n  ```\n\nSee `Ecspanse.Template.Component.Timer` for more details.","ref":"Ecspanse.Template.Event.Timer.html#module-example"},{"type":"behaviour","title":"Ecspanse.Projection","doc":"The `Ecspanse.Projection` behaviour is used to build state projections.\nThe projections are defined by invoking `use Ecspanse.Projection` in their module definition.\n\nProjections are used to build models and query the state of application across multiple\nentities and their components.\n\nThey are designed to be created and used by external clients (such as UI libraries, for example Phoenix LiveView),\n\nThe Projections are GenServers and the client that creates them is responsible for\nstoring their `pid` and using it to communicate with them.\n\nThe module invoking `use Ecspanse.Projection` must implement the mandatory\n`c:Ecspanse.Projection.project/1` callback. This is responsible for\nquerying the state and building the projection struct.\n\n> ##","ref":"Ecspanse.Projection.html"},{"type":"behaviour","title":"Note  {: .warning} - Ecspanse.Projection","doc":"> The `project/2` callback runs every frame, after executing all systems.\n> Many projections with complex queries can have a negative impact on performance.","ref":"Ecspanse.Projection.html#module-note-warning"},{"type":"behaviour","title":"Options - Ecspanse.Projection","doc":"- `:fields` - a list with all the event struct keys and their initial values (if any)\nFor example: `[:pos_x, :pos_y, resources_gold: 0, resources_gems: 0]`","ref":"Ecspanse.Projection.html#module-options"},{"type":"behaviour","title":"Examples - Ecspanse.Projection","doc":"#","ref":"Ecspanse.Projection.html#module-examples"},{"type":"behaviour","title":"The Projection - Ecspanse.Projection","doc":"```elixir\n  defmodule Demo.Projections.Hero do\n    use Ecspanse.Projection, fields: [:pos_x, :pos_y, :resources_gold, :resources_gems]\n\n    @impl true\n    def project(%{entity_id: entity_id} = _attrs) do\n      {:ok, entity} = fetch_entity(entity_id)\n      {:ok, pos} = Demo.Components.Position.fetch(entity)\n      {:ok, gold} = Demo.Components.Gold.fetch(entity)\n      {:ok, gems} = Demo.Components.Gems.fetch(entity)\n\n      struct!(__MODULE__, pos_x: pos.x, pos_y: pos.y, resources_gold: gold.amount, resources_gems: gems.amount)\n    end\n\n    @impl true\n    def on_change(%{client_pid: pid} = _attrs, new_projection, _previous_projection) do\n      # when the projection changes, send it to the client\n      send(pid, {:projection_updated, new_projection})\n    end\n  end\n  ```\n#","ref":"Ecspanse.Projection.html#module-the-projection"},{"type":"behaviour","title":"The Client - Ecspanse.Projection","doc":"```elixir\n  #...\n  projection_pid = Demo.Projections.Hero.start!(%{entity_id: entity.id, client_pid: self()})\n\n  projection = Demo.Projections.Hero.get!(projection_pid)\n\n  # ...\n  def handle_info({:projection_updated, projection}, state) do\n    # received every time the projection changes\n    # ...\n  end\n\n  # ...\n  Demo.Projections.Hero.stop(projection_pid)\n  ```","ref":"Ecspanse.Projection.html#module-the-client"},{"type":"callback","title":"Ecspanse.Projection.get!/1","doc":"Gets the projection struct by providing the server `pid`.\n\n> ##","ref":"Ecspanse.Projection.html#c:get!/1"},{"type":"callback","title":"Implemented Callback  {: .tip} - Ecspanse.Projection.get!/1","doc":"> This callback is implemented by the library and can be used as such.","ref":"Ecspanse.Projection.html#c:get!/1-implemented-callback-tip"},{"type":"callback","title":"Examples - Ecspanse.Projection.get!/1","doc":"```elixir\n  %Demo.Projection.Hero{} = Demo.Projections.Hero.get!(projection_pid)\n  ```","ref":"Ecspanse.Projection.html#c:get!/1-examples"},{"type":"callback","title":"Ecspanse.Projection.on_change/3","doc":"Optional callback that is executed every time the projection changes.\n\nIt takes the `attrs` map argument passed to `c:Ecspanse.Projection.start!/1`,\nthe new projection and the previous projection structs as arguments. The return value is ignored.","ref":"Ecspanse.Projection.html#c:on_change/3"},{"type":"callback","title":"Examples - Ecspanse.Projection.on_change/3","doc":"```elixir\n  @impl true\n  def on_change(%{client_pid: pid} = _attrs, new_projection, _previous_projection) do\n    send(pid, {:projection_updated, new_projection})\n  end\n  ```","ref":"Ecspanse.Projection.html#c:on_change/3-examples"},{"type":"callback","title":"Ecspanse.Projection.project/1","doc":"The `project/1` callback is responsible for querying the state and building the projection struct.\n\nIt takes the `attrs` map argument passed to `c:Ecspanse.Projection.start!/1`.\nIt must return the projection struct.","ref":"Ecspanse.Projection.html#c:project/1"},{"type":"callback","title":"Examples - Ecspanse.Projection.project/1","doc":"```elixir\n    @impl true\n    def project(%{entity_id: entity_id} = _attrs) do\n      {:ok, entity} = fetch_entity(entity_id)\n      {:ok, pos} = Demo.Components.Position.fetch(entity)\n      {:ok, gold} = Demo.Components.Gold.fetch(entity)\n      {:ok, gems} = Demo.Components.Gems.fetch(entity)\n\n      struct!(__MODULE__, pos_x: pos.x, pos_y: pos.y, resources_gold: gold.amount, resources_gems: gems.amount)\n    end\n  ```","ref":"Ecspanse.Projection.html#c:project/1-examples"},{"type":"callback","title":"Ecspanse.Projection.start!/1","doc":"Starts a new projection server and returns its `pid`.\n\nIt takes a single attrs `map` argument.\n\n> ##","ref":"Ecspanse.Projection.html#c:start!/1"},{"type":"callback","title":"Info  {: .info} - Ecspanse.Projection.start!/1","doc":"> The `attrs` map is passed to the `c:Ecspanse.Projection.project/1`\n> and `c:Ecspanse.Projection.on_change/3` callbacks.\n\nThe caller is responsible for storing the returned `pid`.\n\n> ##","ref":"Ecspanse.Projection.html#c:start!/1-info-info"},{"type":"callback","title":"Implemented Callback  {: .tip} - Ecspanse.Projection.start!/1","doc":"> This callback is implemented by the library and can be used as such.","ref":"Ecspanse.Projection.html#c:start!/1-implemented-callback-tip"},{"type":"callback","title":"Examples - Ecspanse.Projection.start!/1","doc":"```elixir\n  projection_pid = Demo.Projections.Hero.start!(%{entity_id: entity.id, client_pid: self()})\n  ```","ref":"Ecspanse.Projection.html#c:start!/1-examples"},{"type":"callback","title":"Ecspanse.Projection.stop/1","doc":"Stops the projection server by its `pid`.\n\n> ##","ref":"Ecspanse.Projection.html#c:stop/1"},{"type":"callback","title":"Implemented Callback  {: .tip} - Ecspanse.Projection.stop/1","doc":"> This callback is implemented by the library and can be used as such.","ref":"Ecspanse.Projection.html#c:stop/1-implemented-callback-tip"},{"type":"callback","title":"Examples - Ecspanse.Projection.stop/1","doc":"```elixir\n  Demo.Projections.Hero.stop(projection_pid)\n  ```","ref":"Ecspanse.Projection.html#c:stop/1-examples"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nThere are a few steps needed to get started with the Ecspanse ECS framework. This guide covers the process of setting up the environment and creating a working application.","ref":"getting_started.html"},{"type":"extras","title":"Create a new Elixir/Phoenix project - Getting Started","doc":"This guide we will use a simple supervised Elixir application:\n\n```bash\nmix new demo --sup\n```","ref":"getting_started.html#create-a-new-elixir-phoenix-project"},{"type":"extras","title":"Installation - Getting Started","doc":"The first step is adding the `:ecspanse` library to the project dependencies. This is done by adding the following line to the `mix.exs` file:\n\n```elixir\ndef deps do\n  [\n    {:ecspanse, \"~> 0.3.0\"}\n  ]\nend\n```","ref":"getting_started.html#installation"},{"type":"extras","title":"Setup - Getting Started","doc":"The subsequent stage involves configuring Ecspanse via `use ECSpanse`. In this guide, the primary Demo module will accommodate the Ecspanse setup:\n\n```elixir\ndefmodule Demo do\n  use Ecspanse\n\n  @impl Ecspanse\n  def setup(data) do\n    data\n  end\nend\n```\n\nThe `c:Ecspanse.setup/1` callback is mandatory. It will be later on used to schedule the application systems.","ref":"getting_started.html#setup"},{"type":"extras","title":"Starting the Ecspanse server - Getting Started","doc":"The module implementing `Ecspanse` needs to be added to the supervision tree.\n\nThe Ecspanse server loop will start together with the application:\n\n```bash\niex -S mix\n```","ref":"getting_started.html#starting-the-ecspanse-server"},{"type":"extras","title":"Tutorial","doc":"# Tutorial\n\nThe objective of this tutorial is to develop a basic game utilizing the Ecspanse framework. The game is the initial stage of an RPG game with minimal features. The implementation focuses on the on the game logic and it does not have a UI. All the user interactions and minimal display will be done via Livebook integration.","ref":"tutorial.html"},{"type":"extras","title":"Story - Tutorial","doc":"The story of the game is simple:\n\n- the game features a single character (Hero).\n- the hero has energy. It starts with 50 energy points and it can have a maximum of 100 energy points. The energy points are used to perform actions. Every 3 seconds the hero restores 1 energy point.\n- the hero can move in four directions in a tiles-like manner, without actually implementing a tile system. For example, if the hero moves right it will transition form (0,0) to (1,0) and so on. Each move costs 1 energy point.\n- on each move the hero has a chance to find resources: gold or gems. Resources are not inventory items, but tradeable items. The hero can trade resources for inventory items.\n- the hero starts with some items in their inventory: 2 potions and one pair of boots.\n- the hero can purchase a map with 2 gold and a compass with 3 gold and 2 gems.\n\nThis setup enables us to delve into fundamental concepts of ECS in general and Ecspanse in particular:\n\n- creating new entities from components\n- querying for components\n- interacting with the system via events\n- scheduling systems to perform actions\n- managing entities relationships\n- different ways to approach collections of entities or components\n- using time-based systems and events\n\n---","ref":"tutorial.html#story"},{"type":"extras","title":"Spawning the Hero - Tutorial","doc":"The goal of this chapter is to spawn the hero entity with its components on game startup.\n\n> #","ref":"tutorial.html#spawning-the-hero"},{"type":"extras","title":"Ecspanse Concepts 1 {: .info} - Tutorial","doc":">\n> - creating components\n> - using commands to spawn entities\n> - creating and scheduling systems\n> - querying components\n\n#","ref":"tutorial.html#ecspanse-concepts-1-info"},{"type":"extras","title":"Adding the Components - Tutorial","doc":"The hero entity will have for now the following components:\n\n```elixir\ndefmodule Demo.Components.Hero do\n  use Ecspanse.Component, state: [name: \"Hero\"]\nend\n\ndefmodule Demo.Components.Energy do\n  use Ecspanse.Component, state: [current: 50, max: 100]\nend\n\ndefmodule Demo.Components.Position do\n  use Ecspanse.Component, state: [x: 0, y: 0]\nend\n\n```\n\nThe `Hero` component holds generic information about the hero. The `Energy` holds the current and maximum energy points. The `Position` component holds the current position of the hero as horizontal and vertical coordinates.\n\nUnder the hood, the components are structs, with some metadata added by the library.\n\nThe following options are available when defining a component:\n\n- `:state` - the fields and the initial state of the component. It should be a list or a keyword list.\n- `:tags` - a list of atoms that can be used to tag the component. Tags are an alternate way of querying components.\n\n#","ref":"tutorial.html#adding-the-components"},{"type":"extras","title":"The Hero Entity Spec - Tutorial","doc":"While this is not mandatory, we extracted the hero entity spec composition in the `Demo.Entities.Hero` module.\n\n```elixir\ndefmodule Demo.Entities.Hero do\n  alias Demo.Components\n\n  @spec new() :: Ecspanse.Entity.entity_spec()\n  def new do\n    {Ecspanse.Entity,\n     components: [\n       Components.Hero,\n       Components.Energy,\n       Components.Position\n     ]}\n  end\nend\n```\n\nThe `new/0` function does not have any effect. It just prepares the entity spec _(of type `t:Ecspanse.Entity.entity_spec/0`)_ to be spawned.\n\n#","ref":"tutorial.html#the-hero-entity-spec"},{"type":"extras","title":"The Spawn Hero System - Tutorial","doc":"The system that spawns the hero is a simple one. It is scheduled to run once, when the game starts.\n\n```elixir\ndefmodule Demo.Systems.SpawnHero do\n  use Ecspanse.System\n\n  @impl true\n  def run(_frame) do\n    %Ecspanse.Entity{} = Ecspanse.Command.spawn_entity!(Demo.Entities.Hero.new())\n  end\nend\n```\n\nThe system must implement the `c:Ecspanse.System.WithoutEventSubscriptions.run/1` or `c:Ecspanse.System.WithEventSubscriptions.run/2` callback. In this case, it is a generic system, not subscribing to any events, so we will use `run/1`. The callback receives the `t:Ecspanse.Frame.t/0` as argument.\n\nOperations that involve the creation of entities or components are done via the functions in the `Ecspanse.Command` module. The commands cannot be executed outside of a system.\n\n#","ref":"tutorial.html#the-spawn-hero-system"},{"type":"extras","title":"Scheduling Spawn Hero the System - Tutorial","doc":"It is now time schedule the newly created system as a startup system. This is done by updating the `c:Ecspanse.setup/1` function in the `Demo` module. We already created this function in the [Getting Started](./getting_started.md) guide.\n\n```elixir\ndefmodule Demo do\n  use Ecspanse\n\n  alias Demo.Systems\n\n  @impl Ecspanse\n  def setup(data) do\n    data\n    |> Ecspanse.add_startup_system(Systems.SpawnHero)\n  end\nend\n```\n\nIf we start the application now, the hero entity will be spawned.\n\n#","ref":"tutorial.html#scheduling-spawn-hero-the-system"},{"type":"extras","title":"Querying for the Hero - Tutorial","doc":"Next, we will incorporate some helper functions that will prove useful in the next chapters.\n\n```elixir\ndefmodule Demo.Entities.Hero do\n  alias Demo.Components\n\n  #...\n\n  def fetch do\n    Ecspanse.Query.select({Ecspanse.Entity}, with: [Components.Hero])\n    |> Ecspanse.Query.one()\n    |> case do\n      {%Ecspanse.Entity{} = entity} -> {:ok, entity}\n      _ -> {:error, :not_found}\n    end\n  end\nend\n```\n\nIn the `Demo.Entities.Hero` module we added the `fetch/0` function. This function uses the `Ecspanse.Query` module to select the hero entity. The `Ecspanse.Query.select/2` function is the most flexible way to query for entities and components and it will be used many times in this tutorial. In the current context, the query can be interpreted as:\n\n- select tuples with a single element, Entity -> this queries the `%Ecspanse.Entity{}` struct itself. When we want to return the entity as part of a more complex select query, it needs to be in the first position of the tuple.\n- that has the `Demo.Components.Hero` component attached to it.\n- return just one record -> this would return the selected entity tuple if found, or otherwise nil. It is important to note that, if many records match the query, it will raise an error. For such cases, the `Ecspanse.Query.stream/1` should be used and it will return a stream of results.\n\nWe can actually test the function in the `iex` console after starting the server:\n\n```iex\niex(1)> Demo.Entities.Hero.fetch()\n{:ok, %Ecspanse.Entity{id: \"e950bf44-16d5-46b5-bd21-85aabae50ce8\"}}\n```\n\nFor the other helper function, we will create an API module. Again, this is not part of the library, but it provides an easy way to interact with the game, no matter what front end we will use.\n\n```elixir\ndefmodule Demo.API do\n  @spec fetch_hero_details() :: {:ok, map()} | {:error, :not_found}\n  def fetch_hero_details do\n    Ecspanse.Query.select(\n      {Demo.Components.Hero, Demo.Components.Energy, Demo.Components.Position}\n    )\n    |> Ecspanse.Query.one()\n    |> case do\n      {hero, energy, position} ->\n        %{name: hero.name, energy: energy.current, max_energy: energy.max, pos_x: position.x, pos_y: position.y}\n      _ ->\n        {:error, :not_found}\n    end\n  end\nend\n```\n\nThis function, returns a map with the hero details we have implemented so far. This time, the `select/2` function is used to select multiple components. The query can be interpreted as:\n\n- select tuples with three elements, `Demo.Components.Hero`, `Demo.Components.Energy`, `Demo.Components.Position` only from entities that have all three components attached to them.\n- return just one record -> this will return a tuple with the three components structs if found, or nil otherwise.\n\nWe can test the function in the `iex` console after starting the server:\n\n```iex\niex(2)> Demo.API.fetch_hero_details()\n%{name: \"Hero\", energy: 50, max_energy: 100, pos_x: 0, pos_y: 0}\n```\n\n---","ref":"tutorial.html#querying-for-the-hero"},{"type":"extras","title":"Hero Movement - Tutorial","doc":"The goal of this chapter is to implement the hero movement. The hero will be able to move in the four directions: up, down, left and right.\n\n> #","ref":"tutorial.html#hero-movement"},{"type":"extras","title":"Ecspanse Concepts 2 {: .info} - Tutorial","doc":">\n> - receiving external input through events\n> - implementing ans scheduling async systems\n> - locking components for parallel operations\n> - implementing systems that subscribe to events\n> - updating components with commands\n\n#","ref":"tutorial.html#ecspanse-concepts-2-info"},{"type":"extras","title":"The Move Event - Tutorial","doc":"Ecspanse receives external input through events. Let's implement the move event.\n\n```elixir\ndefmodule Demo.Events.MoveHero do\n  use Ecspanse.Event, fields: [:direction]\nend\n```\n\nSimilar to the components, the events are structs under the hood. The fields and their default values are defined with the `:fileds` option.\n\nWe can also expose the event in the API module:\n\n```elixir\ndefmodule Demo.API do\n  #...\n  @spec move_hero(direction :: :up | :down | :left | :right) :: :ok\n  def move_hero(direction) do\n    Ecspanse.event({Demo.Events.MoveHero, direction: direction})\n  end\nend\n```\n\nWe create another event that will be emitted when the hero actually moved to handle various side effects.\n\n```elixir\ndefmodule Demo.Events.HeroMoved do\n  use Ecspanse.Event\nend\n```\n\n#","ref":"tutorial.html#the-move-event"},{"type":"extras","title":"The Move System - Tutorial","doc":"The role of the move system is to listen to move events, then check if the hero has enough energy to move and if so, update the hero position and adjust the energy. We want this system to run asynchronously.\n\n```elixir\ndefmodule Demo.Systems.MoveHero do\n  use Ecspanse.System,\n    lock_components: [Demo.Components.Position, Demo.Components.Energy],\n    event_subscriptions: [Demo.Events.MoveHero]\n\n  alias Demo.Components\n\n  @impl true\n  def run(%Demo.Events.MoveHero{direction: direction}, _frame) do\n    components =\n      Ecspanse.Query.select({Components.Position, Components.Energy}, with: [Components.Hero])\n      |> Ecspanse.Query.one()\n\n    with {position, energy}  = 1 do\n      :ok\n    else\n      {:error, :not_enough_energy}\n    end\n  end\n\n  defp update_coordinates(%Components.Position{x: x, y: y}, direction) do\n    case direction do\n      :up -> [x: x, y: y + 1]\n      :down -> [x: x, y: y - 1]\n      :left -> [x: x - 1, y: y]\n      :right -> [x: x + 1, y: y]\n      _ -> [x: x, y: y]\n    end\n  end\nend\n```\n\n##","ref":"tutorial.html#the-move-system"},{"type":"extras","title":"Component Locking - Tutorial","doc":"We said that the move system will run asynchronously. This means that it will run in parallel with the other systems. The `lock_components` option is used to specify the components that will be locked by the system. That means that no other systems that lock at least one of the locket components will run in the same parallel batch as the `MoveHero` system. In our case, we want to lock the `Demo.Components.Position` and `Demo.Components.Energy` components. This is because we want to update the hero position and energy, and we need to avoid race conditions.\n\nThe commands will check if the system is async and will raise an error if we try to update, insert or delete components that are not locked. For extra safety we can also lock components for which we don't update the state, but we read and depend on it.\n\n##","ref":"tutorial.html#component-locking"},{"type":"extras","title":"Event Subscriptions - Tutorial","doc":"Not all systems are required to run every single frame. The `MoveHero` system is useful only when a `MoveHero` event is received. The `event_subscriptions` option is used to specify the events that the system is interested in. The system will run only when at least one of the subscribed events is received.\n\nThe systems that have events subscriptions need to implement the `run/2` callback. The first argument is the event that triggered the system. The second argument is the current frame.\n\n##","ref":"tutorial.html#event-subscriptions"},{"type":"extras","title":"Updating the Components - Tutorial","doc":"It is a good practice to make sure that actions can be performed before committing any updates. Reverting the changes would be more difficult and inefficient.\n\nIn our case, we want to make sure that the hero has enough energy before updating the `Energy` and `Position` components state.\n\n#","ref":"tutorial.html#updating-the-components"},{"type":"extras","title":"Scheduling the Move System - Tutorial","doc":"We use `add_system/2` to schedule the `MoveHero` system to run asynchronously.\n\n```elixir\ndefmodule Demo do\n  #...\n  def setup(data) do\n    data\n    #...\n    |> Ecspanse.add_system(Systems.MoveHero)\n  end\nend\n```\n\nWe can try the hero movement in the `iex` console:\n\n```iex\niex(1)> Demo.API.fetch_hero_details()\n%{name: \"Hero\", energy: 50, max_energy: 100, pos_x: 0, pos_y: 0}\niex(2)> Demo.API.move_hero(:up)\n:ok\niex(3)> Demo.API.move_hero(:right)\n:ok\niex(4)> Demo.API.fetch_hero_details()\n%{name: \"Hero\", energy: 48, max_energy: 100, pos_x: 1, pos_y: 1}\n```\n\n---","ref":"tutorial.html#scheduling-the-move-system"},{"type":"extras","title":"Energy Regeneration - Tutorial","doc":"The goal of this chapter is to implement the energy regeneration. The hero will restore 1 point of energy every 3 seconds.\n\n> #","ref":"tutorial.html#energy-regeneration"},{"type":"extras","title":"Ecspanse Concepts 3 {: .info} - Tutorial","doc":">\n> - using the timer to schedule events at precise intervals\n> - using built-in component and event templates\n> - ordering async systems\n> - conditionally running systems\n> - new ways of querying entities and components\n\n#","ref":"tutorial.html#ecspanse-concepts-3-info"},{"type":"extras","title":"The Energy Timer Component - Tutorial","doc":"Timer-based components use the provided `Ecspanse.Template.Component.Timer` component template. The timer component is a special component that is used to schedule events at precise intervals. The timer component template exposes the following fields:\n\n- `:duration` - the countdown duration in milliseconds\n- `:time` - the current countdown time in milliseconds\n- `:event` - the event that will be triggered when the countdown reaches 0\n- `:mode` - `:repeat | :once | :temporary` - decides the timer behavior after the countdown reaches 0.\n- `:paused` - `boolean` - can be used to pause the timer\n\nWe will discuss more about template components in the next chapter.\n\n```elixir\ndefmodule Demo.Components.EnergyTimer do\n  use Ecspanse.Template.Component.Timer,\n    state: [duration: 3000, time: 3000, event: Demo.Events.EnergyTimerFinished, mode: :repeat]\nend\n```\n\nWe add the new component to the Hero entity:\n\n```elixir\ndefmodule Demo.Entities.Hero do\n  #...\n  @spec new() :: Ecspanse.Entity.entity_spec()\n  def new do\n    {Ecspanse.Entity,\n    components: [\n      Components.Hero,\n      Components.Energy,\n      Components.Position,\n      Components.EnergyTimer\n    ]}\n  end\n  #...\nend\n```\n\n#","ref":"tutorial.html#the-energy-timer-component"},{"type":"extras","title":"The Energy Timer Finished Event - Tutorial","doc":"Timer-based events use the provided `Ecspanse.Template.Event.Timer` event template. The timer event template exposes the following fields:\n\n- `entity_id` - the id of the entity that owns the timer component\n\n```elixir\ndefmodule Demo.Events.EnergyTimerFinished do\n  use Ecspanse.Template.Event.Timer\nend\n```\n\nThis event will be automatically triggered when the EnergyTimer component duration reaches 0.\n\n#","ref":"tutorial.html#the-energy-timer-finished-event"},{"type":"extras","title":"The Energy Restore System - Tutorial","doc":"```elixir\ndefmodule Demo.Systems.RestoreEnergy do\n  use Ecspanse.System,\n    lock_components: [Demo.Components.Energy],\n    event_subscriptions: [Demo.Events.EnergyTimerFinished]\n\n  @impl true\n  def run(%Demo.Events.EnergyTimerFinished{entity_id: entity_id}, _frame) do\n    with {:ok, entity} <- Ecspanse.Query.fetch_entity(entity_id),\n         {:ok, energy} <- Ecspanse.Query.fetch_component(entity, Demo.Components.Energy) do\n      Ecspanse.Command.update_component!(energy, current: energy.current + 1)\n    end\n  end\nend\n```\n\nThe system locks the `Energy` component to update its state. And subscribes to the `EnergyTimerFinished` event because it is interested only in that timer event.\n\nThe system adds 1 point to the current energy. You are right to wonder why don't we first check the max energy cap. We will clarify this in the next section.\n\nThis system introduces also new ways of querying entities and components: `Ecspanse.Query.fetch_entity/1` and `Ecspanse.Query.fetch_component/2`.\n\n**TIP**\nThe following functions would produce the same results:\n\n```elixir\nEcspanse.Query.fetch_component(entity, Demo.Components.Energy)\n#and\nDemo.Components.Energy.fetch(entity)\n```\n\n#","ref":"tutorial.html#the-energy-restore-system"},{"type":"extras","title":"Rescheduling the Systems Execution - Tutorial","doc":"It is time to re-write the Demo module:\n\n```elixir\ndefmodule Demo do\n  use Ecspanse\n\n  alias Demo.Systems\n\n  @impl Ecspanse\n  def setup(data) do\n    data\n    |> Ecspanse.add_startup_system(Systems.SpawnHero)\n    |> Ecspanse.add_system(Systems.RestoreEnergy, run_if: [{__MODULE__, :energy_not_max}])\n    |> Ecspanse.add_system(Systems.MoveHero, run_after: [Systems.RestoreEnergy])\n    |> Ecspanse.add_frame_end_system(Ecspanse.System.Timer)\n  end\n\n  def energy_not_max do\n    Ecspanse.Query.select({Demo.Components.Energy}, with: [Demo.Components.Hero])\n    |> Ecspanse.Query.one()\n    |> case do\n      {%Demo.Components.Energy{current: current, max: max}} ->\n        current  \n        false\n    end\n  end\nend\n\n```\n\n##","ref":"tutorial.html#rescheduling-the-systems-execution"},{"type":"extras","title":"The Conditional System Execution - Tutorial","doc":"By using the `:run_if` option, the `RestoreEnergy` system will run only if the current energy is below the max energy. The `energy_not_max/0` function must always return a boolean value. Please note, this is not an efficient implementation. The `energy_not_max/0` function will be called every frame. If the check would happen in the `RestoreEnergy` system, it would run only once every 3 seconds. But we took the opportunity to exemplify conditionally running systems.\n\n##","ref":"tutorial.html#the-conditional-system-execution"},{"type":"extras","title":"The System Execution Order - Tutorial","doc":"By using the `:after` option, the `MoveHero` system will run after the `RestoreEnergy` system. Both are async systems, but even the async systems run in batches, not all at once. The batches are scheduled depending on the locked components and the specified order of execution of the systems.\n\n> ##","ref":"tutorial.html#the-system-execution-order"},{"type":"extras","title":"Note {: .info} - Tutorial","doc":">\n> It does not matter if the `RestoreEnergy` system actually runs this turn.\n> The `MoveHero` will still run if receiving the `MoveHero` event.\n> The `:run_after` option, just guarantees that if both systems are running,\n> the `MoveHero` will run after the `RestoreEnergy`.\n\n##","ref":"tutorial.html#note-info"},{"type":"extras","title":"Scheduling the Built-in Timer System - Tutorial","doc":"Once we start using timer-based components, the built-in `Ecspanse.System.Timer` system must be scheduled to run synchronously at the beginning or at the end of every frame. It will update all the timer-based components and trigger the timer events.\n\n---","ref":"tutorial.html#scheduling-the-built-in-timer-system"},{"type":"extras","title":"Finding Resources - Tutorial","doc":"The goal of this chapter is to implement the resource gathering. With each move, the hero has a chance to find gold or gems.\n\n> #","ref":"tutorial.html#finding-resources"},{"type":"extras","title":"Ecspanse Concepts 4 {: .info} - Tutorial","doc":">\n> - using tags to manage collections of components\n> - using advanced component specs\n> - using component templates\n> - using the auto-emitted component updatd events\n\n#","ref":"tutorial.html#ecspanse-concepts-4-info"},{"type":"extras","title":"Creating the Resource Template Component - Tutorial","doc":"Template components are used to define the structure for related components. It is a guarantee that certain componets will have certain fields in their state.\n\nTogether with tags, this is a powerful way to achieve polymorphism in components.\n\n```elixir\ndefmodule Demo.Components.Resource do\n  use Ecspanse.Template.Component, state: [:id, :name, amount: 0], tags: [:resource]\n\n  @impl true\n  def validate(state) do\n    with :ok  = 0 do\n      :ok\n    else\n      {:error, \"#{inspect(amount)} must be positive\"}\n    end\n  end\nend\n```\n\nPlease note that the template `c:Ecspanse.Template.Component.validate/1` callback is optional. It runs only at compile time and it takes the list of state fields as argument.\n\n#","ref":"tutorial.html#creating-the-resource-template-component"},{"type":"extras","title":"Creating the Resource Components - Tutorial","doc":"```elixir\ndefmodule Demo.Components.Gems do\n  use Demo.Components.Resource,\n    state: [id: :gems, name: \"Gems\", amount: 0], tags: [:resource]\nend\n\ndefmodule Demo.Components.Gold do\n  use Demo.Components.Resource,\n    state: [id: :gems, name: \"Gold\", amount: 0], tags: [:resource]\nend\n```\n\nAs you can observe, the two components are invoking the newly defined template with `use Demo.Components.Resource` instead of `use Ecspanse.Component`.\n\nAnother new concept introduced both here and in the templat definition is the `:tags` option. It is a list of atoms that can be used to group and query components. The resource components can now be used as a resource store for the user, but they can also be used to represent the cost of various items. We will handle the second use case in the next chapters.\n\nFor such cases, it is important to use a standardized approach, a perfect use-case for templates. Eg. all the resource components should have the same state fields.\n\n#","ref":"tutorial.html#creating-the-resource-components"},{"type":"extras","title":"Adding the Resources Components to the Hero Entity - Tutorial","doc":"```elixir\ndefmodule Demo.Entities.Hero do\n  #...\n\n  def new do\n    {Ecspanse.Entity,\n     components: [\n        #...\n       {Components.Gold, [], [:available]},\n       {Components.Gems, [], [:available]}\n     ]}\n  end\n  #...\nend\n```\n\nWe use the `t:Ecspanse.Component.component_spec/0` type to specify the component spec. The first element of the tuple is the component module, the second element is the initial state of the component, and the third element is a list of tags.\n\nThe initial state of the component can be changed at runtime like `{Components.Gold, [amount: 5], [:available]}`. Also, new tags can be added at the time of the component creation. They will be appended to the list defined in the component module.\n\nRuntime tag setting enables the reusability of components in various scenarios. It's important to note that changing a component's tags after it has been created is not supported.\n\n#","ref":"tutorial.html#adding-the-resources-components-to-the-hero-entity"},{"type":"extras","title":"Storing Found Resources - Tutorial","doc":"Our new `MaybeFindResources` system subscribes to `Demo.Events.HeroMoved` emitted by the `MoveHero` system. Then it randomly decides if the current position contains resources, and the type of resource.\n\n```elixir\ndefmodule Demo.Systems.MaybeFindResources do\n  use Ecspanse.System,\n    lock_components: [Demo.Components.Gems, Demo.Components.Gold],\n    event_subscriptions: [Demo.Events.HeroMoved]\n\n  alias Demo.Components\n\n  @impl true\n  def run(%Ecspanse.Event.HeroMoved{}, _frame) do\n    with true   Ecspanse.add_system(Systems.MoveHero, run_after: [Systems.RestoreEnergy])\n    |> Ecspanse.add_system(Systems.MaybeFindResources)\n    |> Ecspanse.add_frame_end_system(Ecspanse.System.Timer)\n  end\nend\n```\n\nThe last step of the current section is to expose the resources in the `fetch_hero_details/0` funciton in the `Demo.API` module.\n\n```elixir\n  defp list_hero_resources(hero_entity) do\n    hero_entity\n    |> Ecspanse.Query.list_tagged_components_for_entity([:resource, :available])\n    |> Enum.map(&%{name: &1.name, amount: &1.amount})\n  end\n```\n\nHere we use the `Ecspanse.Query.list_tagged_components_for_entity/2` function to get all the components tagged with `:resource` and `:available` for the hero entity.\n\nStarting the application and moving the hero around will now start to accumulate resources:\n\n```iex\niex(14)> Demo.API.fetch_hero_details\n%{\n  name: \"Hero\",\n  resources: [%{name: \"Gems\", amount: 2}, %{name: \"Gold\", amount: 5}],\n  energy: 56,\n  max_energy: 100,\n  pos_x: -3,\n  pos_y: -5\n}\n```\n\n--","ref":"tutorial.html#storing-found-resources"},{"type":"extras","title":"Market and Inventory Items - Tutorial","doc":"The gloal of this chapter is to implement invetory items and a market. The hero can buy items from the market with resources and store them in the inventory.\n\n> #","ref":"tutorial.html#market-and-inventory-items"},{"type":"extras","title":"Ecspanse Concepts 5 {: .info} - Tutorial","doc":">\n> - using relationships to manage collections of entities\n> - querying components within entities relationships\n\n#","ref":"tutorial.html#ecspanse-concepts-5-info"},{"type":"extras","title":"Inventory Items Components and Entities Specs - Tutorial","doc":"We start by defining the inventory items and the market components:\n\n```elixir\ndefmodule Demo.Components.Market do\n  use Ecspanse.Component\nend\n\ndefmodule Demo.Components.Boots do\n  use Ecspanse.Component, state: [name: \"Boots\"], tags: [:inventory]\nend\n\ndefmodule Demo.Components.Compass do\n  use Ecspanse.Component, state: [name: \"Compass\"], tags: [:inventory]\nend\n\ndefmodule Demo.Components.Map do\n  use Ecspanse.Component, state: [name: \"Map\"], tags: [:inventory]\nend\n\ndefmodule Demo.Components.Potion do\n  use Ecspanse.Component, state: [name: \"Potion\"], tags: [:inventory]\nend\n```\n\nThe inventory items, however are more complex than this. They cost resources, and in the future they may have various attributes impacting the hero abilities. So the items will be entities of their own. We will create a new `Entities.Inventory` module to manage the inventory items specs.\n\n```elixir\ndefmodule Demo.Entities.Inventory do\n  alias Demo.Components\n\n  @spec new_boots() :: Ecspanse.Entity.entity_spec()\n  def new_boots do\n    {Ecspanse.Entity, components: [Components.Boots, {Components.Gold, [amount: 3], [:cost]}]}\n  end\n\n  @spec new_compass() :: Ecspanse.Entity.entity_spec()\n  def new_compass do\n    {Ecspanse.Entity,\n     components: [\n       Components.Compass,\n       {Components.Gold, [amount: 3], [:cost]},\n       {Components.Gems, [amount: 2], [:cost]}\n     ]}\n  end\n\n  @spec new_map() :: Ecspanse.Entity.entity_spec()\n  def new_map do\n    {Ecspanse.Entity, components: [Components.Map, {Components.Gold, [amount: 2], [:cost]}]}\n  end\n\n  @spec new_potion() :: Ecspanse.Entity.entity_spec()\n  def new_potion do\n    {Ecspanse.Entity, components: [Components.Potion, {Components.Gold, [amount: 1], [:cost]}]}\n  end\n\n  @spec list_inventory_components(Ecspanse.Entity.t()) :: [component :: struct()]\n  def list_inventory_components(parent) do\n    Ecspanse.Query.list_tagged_components_for_children(parent, [:inventory])\n  end\nend\n```\n\nEach item is defined together with their cost in resources. Please note that now we are using the `:cost` tag to mark the cost resource components.\n\nThe `list_inventory_components/1` function is used to list all the inventory items for a given parent entity. We will see what this means in the next section.\n\n#","ref":"tutorial.html#inventory-items-components-and-entities-specs"},{"type":"extras","title":"Inventory Items as Children Entities - Tutorial","doc":"Let's start by updating the existing `SpawnHero` system.\n\n```elixir\ndefmodule Demo.Systems.SpawnHero do\n  use Ecspanse.System\n\n  @impl true\n  def run(_frame) do\n    hero_entity = %Ecspanse.Entity{} = Ecspanse.Command.spawn_entity!(Demo.Entities.Hero.new())\n    potion_entity_1 = %Ecspanse.Entity{} = Ecspanse.Command.spawn_entity!(Demo.Entities.Inventory.new_potion())\n    potion_entity_2 = %Ecspanse.Entity{} = Ecspanse.Command.spawn_entity!(Demo.Entities.Inventory.new_potion())\n    boots_entity = %Ecspanse.Entity{} = Ecspanse.Command.spawn_entity!(Demo.Entities.Inventory.new_boots())\n\n    Ecspanse.Command.add_children!([ {hero_entity, [potion_entity_1, potion_entity_2, boots_entity]} ])\n  end\nend\n```\n\nThe hero starts the journey with two potions and a pair of boots. We use the `Ecspanse.Command.add_children!/1` function to add the inventory items as children of the hero entity. This way we can build complex entities by composing smaller entities.\n\nThe Ecspanse library provides many helper functions to query and change entities relations.\n\nThe next step is to create a new system that spawns a market entity that holds more items.\n\n```elixir\ndefmodule Demo.Systems.SpawnMarket do\n  use Ecspanse.System\n\n  @impl true\n  def run(_frame) do\n    compass_entity = %Ecspanse.Entity{} = Ecspanse.Command.spawn_entity!(Demo.Entities.Inventory.new_compass())\n    map_entity = %Ecspanse.Entity{} = Ecspanse.Command.spawn_entity!(Demo.Entities.Inventory.new_map())\n\n    Ecspanse.Command.spawn_entity!({ Ecspanse.Entity,\n      components: [Demo.Components.Market], children: [compass_entity, map_entity]\n    })\n  end\nend\n```\n\nThis shows another way to spawn an entity with children already attached. The new system needs to be added to the `setup/1` as startup system:\n\n```elixir\n#...\n|> Ecspanse.add_startup_system(Systems.SpawnMarket)\n#...\n```\n\nOne last thing we can do in this chapter is to add new funtions to our API:\n\n```elixir\ndefmodule Demo.API do\n  #...\n  defp list_hero_inventory(hero_entity) do\n    hero_entity\n    |> Demo.Entities.Inventory.list_inventory_components()\n    |> Enum.map(&%{name: &1.name})\n  end\n\n  @spec fetch_market_items() :: {:ok, list(map())} | {:error, :not_found}\n  def fetch_market_items do\n    Ecspanse.Query.select({Ecspanse.Entity}, with: [Demo.Components.Market])\n    |> Ecspanse.Query.one()\n    |> case do\n      {market_entity} -> list_market_items(market_entity)\n      _ -> {:error, :not_found}\n    end\n  end\n\n  defp list_market_items(market_entity) do\n    market_entity\n    |> Demo.Entities.Inventory.list_inventory_components()\n    |> Enum.map(fn item_component ->\n      item_entity = Ecspanse.Query.get_component_entity(item_component)\n\n      %{entity_id: item_entity.id, name: item_component.name, cost: item_cost(item_entity)}\n    end)\n  end\n\n  defp item_cost(item_entity) do\n    item_entity\n    |> Ecspanse.Query.list_tagged_components_for_entity([:resource, :cost])\n    |> Enum.map(&%{name: &1.name, amount: &1.amount})\n  end\nend\n```\n\nWe will now display the hero's inventory and the market items with their respective prices.\n\nWe can test the new functions in the `iex` console:\n\n```iex\niex(1)> Demo.API.fetch_hero_details()\n%{\n  name: \"Hero\",\n  resources: [%{name: \"Gems\", amount: 0}, %{name: \"Gold\", amount: 0}],\n  inventory: [%{name: \"Boots\"}, %{name: \"Potion\"}, %{name: \"Potion\"}],\n  energy: 60,\n  max_energy: 100,\n  pos_x: 0,\n  pos_y: 0\n}\n\niex(1)> Demo.API.fetch_market_items()\n[\n  %{\n    name: \"Map\",\n    entity_id: \"361c00ba-4dd3-4be8-b171-00e99c0b8ef7\",\n    cost: [%{name: \"Gold\", amount: 2}]\n  },\n  %{\n    name: \"Compass\",\n    entity_id: \"b027fd01-d4fe-4ac1-9736-6b6f8c58fbd1\",\n    cost: [%{name: \"Gold\", amount: 3}, %{name: \"Gems\", amount: 2}]\n  }\n]\n```\n\n---","ref":"tutorial.html#inventory-items-as-children-entities"},{"type":"extras","title":"Purchasing Items from the Market - Tutorial","doc":"The goal of this chapter is to allow the hero to purchase items from the market using resources.\n\n> #","ref":"tutorial.html#purchasing-items-from-the-market"},{"type":"extras","title":"Ecspanse Concepts 6 {: .info} - Tutorial","doc":">\n> - in-depth entity relationships\n\n#","ref":"tutorial.html#ecspanse-concepts-6-info"},{"type":"extras","title":"Purchasing Items Event - Tutorial","doc":"The event that triggeres an item purchase is very simple:\n\n```elixir\ndefmodule Demo.Events.PurchaseMarketItem do\n  use Ecspanse.Event, fields: [:item_entity_id]\nend\n```\n\nIt stores only the ID of the entity of the item being purchased.\nOn the other hand, the system is a bit more comlex.\n\n#","ref":"tutorial.html#purchasing-items-event"},{"type":"extras","title":"Purchasing Items System - Tutorial","doc":"Let's start with the code:\n\n```elixir\ndefmodule Demo.Systems.PurchaseMarketItem do\n  use Ecspanse.System, event_subscriptions: [Demo.Events.PurchaseMarketItem]\n\n  @impl true\n  def run(%Demo.Events.PurchaseMarketItem{item_entity_id: item_entity_id}, _frame) do\n    with {:ok, item_entity}   Ecspanse.Query.one()\n    |> case do\n      {market_entity} -> {:ok, market_entity}\n      _ -> {:error, :not_found}\n    end\n  end\n\n  defp has_enough_resources?(available_resources, cost_resources) do\n    Enum.all?(cost_resources, fn cost_resource ->\n      Enum.any?(available_resources, fn available_resource ->\n        available_resource.id == cost_resource.id &&\n          available_resource.amount >= cost_resource.amount\n      end)\n    end)\n  end\n\n  defp spend_resources(available_resources, cost_resources) do\n    Enum.each(cost_resources, fn cost_resource ->\n      available_resource =\n        Enum.find(available_resources, fn available_resource ->\n          available_resource.id == cost_resource.id\n        end)\n\n      Ecspanse.Command.update_component!(available_resource,\n        amount: available_resource.amount - cost_resource.amount\n      )\n    end)\n  end\nend\n```\n\nThis system modifies many components, so one option is to make it synchronous. This way we don't have to individually lock each modified component. Later on it can be refactored into async if needed.\n\nBefore commiting any component state changes it is important to perform all the required validations.\n\n##","ref":"tutorial.html#purchasing-items-system"},{"type":"extras","title":"Validate Entities Exist - Tutorial","doc":"First of all, we want to make sure that the affected entities still exist. We use the `Ecspanse.Query.fetch_entity/1` function to validate that the item entity exists. For the market entity we implement a custom query, while for the hero, we use the helper function we created earlier.\n\n##","ref":"tutorial.html#validate-entities-exist"},{"type":"extras","title":"Validate Relationships - Tutorial","doc":"We need to make sure that the item we want to purchase is still available in the market. In a multiplayer game scenario this would avoid race conditions where two players purchase the same item in the same time. We use the `Ecspanse.Query.is_child_of?/1` function to validate that the item is still a child of the market.\n\n##","ref":"tutorial.html#validate-relationships"},{"type":"extras","title":"Vlaidate Resources - Tutorial","doc":"Before the purchase is made, we need to make sure that the hero has enough resources to buy the item. Again, the tags prove useful. They allow us to query the same components from different entities and compare them.\n\n##","ref":"tutorial.html#vlaidate-resources"},{"type":"extras","title":"Spending the Resources - Tutorial","doc":"Once all the validations are done, the resources can be spent. We iterate through the costs, then reduce the amount of the corresponding available resource.\n\n##","ref":"tutorial.html#spending-the-resources"},{"type":"extras","title":"Changing the Item Entity Parent - Tutorial","doc":"Finally, we remove the item from the market and add it to the hero's inventory. For this, we use the `Ecspanse.Command.remove_child!/2` and `Ecspanse.Command.add_child!/2` functions.\n\n#","ref":"tutorial.html#changing-the-item-entity-parent"},{"type":"extras","title":"The Purchase API - Tutorial","doc":"We first need to add the `PurchaseMarketItem` event to setup as sync system:\n\n```elixir\n#...\n|> Ecspanse.add_frame_end_system(Systems.PurchaseMarketItem)\n#...\n```\n\nThen expose the event in the API:\n\n```elixir\n@spec purchase_market_item(item_entity_id :: Ecspanse.Entity.id()) :: :ok\ndef purchase_market_item(item_entity_id) do\n  Ecspanse.event({Demo.Events.PurchaseMarketItem, item_entity_id: item_entity_id})\nend\n```\n\nNow we can test it in the console. First make sure that the hero has enough resources by walking around and using the exposed `fetch_hero_details/0` function. Then check the market items with `fetch_market_items/0` and note down the desired item entity ID. Purchase the item with `purchase_market_item/1`. Finally, check the hero details again to see the item in the inventory.\n\n---","ref":"tutorial.html#the-purchase-api"},{"type":"extras","title":"Testing the Systems - Tutorial","doc":"The goal of this chapter is to learn how to test the systems. We will use the `MoveHero` system as an example.\n\n> #","ref":"tutorial.html#testing-the-systems"},{"type":"extras","title":"Ecspanse Concepts 7 {: .info} - Tutorial","doc":">\n> - testing systems in isolation\n> - using a custom Ecspanse setup\n> - using the system debugger to run systems manually\n\nThe game story is now ready. Time to see how we can test it.\n\nBefore we start, it is important to note that in test mode, the `Ecspanse.Server` is not automatically started. This allows us to decide the moment when the server should start, and what systems to run.\n\nThere are many ways to test systems. We will choose the most straightforward for this tutorial: testing the systems in isolation. That means that the systems scheduled under `Demo.setup/1` are not running. We will create a new setup function for testing, and call the systems manually.\n\n```elixir\ndefmodule Demo.Systems.MoveHeroTest do\n  use ExUnit.Case, async: false\n\n  defmodule DemoTest do\n    use Ecspanse\n    @impl true\n    def setup(data) do\n      data\n    end\n  end\n\n  setup do\n    {:ok, _pid} = start_supervised({DemoTest, :test})\n    Ecspanse.System.debug()\n\n    hero_entity = %Ecspanse.Entity{} = Ecspanse.Command.spawn_entity!(Demo.Entities.Hero.new())\n    {:ok, position_component} = Demo.Components.Position.fetch(hero_entity)\n\n    assert position_component.x == 0\n    assert position_component.y == 0\n\n    {:ok, energy_component} = Demo.Components.Energy.fetch(hero_entity)\n    assert energy_component.current == 50\n\n    {:ok, hero_entity: hero_entity, energy_component: energy_component}\n  end\n\n  test \"hero moves if enough energy\", %{hero_entity: hero_entity} do\n    event = move(:up)\n    frame = frame(event)\n    Demo.Systems.MoveHero.run(event, frame)\n\n    {:ok, position_component} = Demo.Components.Position.fetch(hero_entity)\n    assert position_component.x == 0\n    assert position_component.y == 1\n\n    {:ok, energy_component} = Demo.Components.Energy.fetch(hero_entity)\n    assert energy_component.current == 49\n\n    #...\n  end\n\n  test \"hero doesn not move if not enough energy\", %{\n    hero_entity: hero_entity,\n    energy_component: energy_component\n  } do\n    Ecspanse.Command.update_component!(energy_component, current: 0)\n\n    event = move(:up)\n    frame = frame(event)\n    Demo.Systems.MoveHero.run(event, frame)\n\n    {:ok, position_component} = Demo.Components.Position.fetch(hero_entity)\n    assert position_component.x == 0\n    assert position_component.y == 0\n\n    {:ok, energy_component} = Demo.Components.Energy.fetch(hero_entity)\n    assert energy_component.current == 0\n  end\n\n  defp move(direction) do\n    %Demo.Events.MoveHero{direction: direction, inserted_at: System.os_time()}\n  end\n\n  defp frame(event) do\n    %Ecspanse.Frame{event_batches: [[event]], delta: 1}\n  end\nend\n```\n\n#","ref":"tutorial.html#ecspanse-concepts-7-info"},{"type":"extras","title":"Test Dependencies - Tutorial","doc":"We start by creating a `DemoTest` module and implement a `setup/1` function that does not schedule any systems.\n\n#","ref":"tutorial.html#test-dependencies"},{"type":"extras","title":"Test Setup - Tutorial","doc":"There are 2 things happening at the top of the `setup` block:\n\n- we manually start the server by passing the `{DemoTest, :test}` tuple to the `start_supervised/1` function. Compared to the normal setup where we add `Demo` to the supervision tree, the `{MODULE, :test}` tuple allows us to start the server in test mode.\n- we run the `Ecspanse.System.debug/0` function. This function \"upgrades\" the current test PID to a system, which allows us to run systems manually. As mentioned previously, commands can be run only from inside a system. Making the test PID a system allows us to run commands directly in our tests.\n\nFor the rest of the setup block we setup the test data. We spawn a hero entity and fetch its position and energy components. We assert that the hero is at the starting position and has the starting energy.\n\n#","ref":"tutorial.html#test-setup"},{"type":"extras","title":"Test the Hero Can Move - Tutorial","doc":"We create two helper functions that would create a `MoveHero` event and a `t:Ecspanse.Frame.t/0` with the event in it.\n\nThen we can run the `MoveHero` system manually by simply calling `Demo.Systems.MoveHero.run(event, frame)`.\n\nFrom there on, we can query any component and do any relevant assertion. In this case, we assert that the hero has moved and that the energy has been reduced.\n\n---","ref":"tutorial.html#test-the-hero-can-move"},{"type":"extras","title":"Running the Demo - Tutorial","doc":"The code for this tutorial, together with instructions on how to run it in Livebook is available on [GitHub](https://github.com/iacobson/ecspanse_demo).\n\nAlso, you can find a more complex example of a multiplayer game built with Ecspanse on [GitHub](https://github.com/iacobson/iveseenthings).","ref":"tutorial.html#running-the-demo"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.7.1 (2023-10-07) - Changelog","doc":"#","ref":"changelog.html#v0-7-1-2023-10-07"},{"type":"extras","title":"Improvements - Changelog","doc":"- `Ecspanse.Command.clone_entity!/2` and `Ecspanse.Command.deep_clone_entity!/2` now accept an `:id` option to set the id of the cloned entity.","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.0 (2023-10-05) - Changelog","doc":"#","ref":"changelog.html#v0-7-0-2023-10-05"},{"type":"extras","title":"Breaking - Changelog","doc":"- `c:Ecspanse.Projection.on_change/3` replaces the `on_change/2` and now takes both the new projection as well as the previous projection as arguments.\n\n#","ref":"changelog.html#breaking"},{"type":"extras","title":"Improvements - Changelog","doc":"- updating projections after all frame systems have run to return a consistent state.","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.6.0 (2023-10-02) - Changelog","doc":"#","ref":"changelog.html#v0-6-0-2023-10-02"},{"type":"extras","title":"Features - Changelog","doc":"- introduces `Ecspanse.Projection` to build state projections across entities and components.","ref":"changelog.html#features"},{"type":"extras","title":"v0.5.0 (2023-09-28) - Changelog","doc":"#","ref":"changelog.html#v0-5-0-2023-09-28"},{"type":"extras","title":"Features - Changelog","doc":"- introduces ancestors queries to query for parents of an entity, the parents of parents, and so on:\n  - `Ecspanse.Query.select/2` new option: `:for_ancestors_of`\n  - `Ecspanse.Query.list_ancestors/1`\n  - `Ecspanse.Query.list_tagged_components_for_ancestors/2`","ref":"changelog.html#features"},{"type":"extras","title":"v0.4.0 (2023-09-17) - Changelog","doc":"#","ref":"changelog.html#v0-4-0-2023-09-17"},{"type":"extras","title":"Breaking - Changelog","doc":"- removed the automatically generated events: `Ecspanse.Event.{ComponentCreated, ComponentUpdated, ComponentDeleted, ResourceCreated, ResourceUpdated, ResourceDeleted}`. Use custom emitted events or short-lived components instead.\n\n#","ref":"changelog.html#breaking"},{"type":"extras","title":"Improvements - Changelog","doc":"- improves performance for tagged components. The system loop now runs faster when dealing with tagged components.","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.3.1 (2023-08-30) - Changelog","doc":"#","ref":"changelog.html#v0-3-1-2023-08-30"},{"type":"extras","title":"Fixes - Changelog","doc":"- fixes a bug where events could be scheduled after they were batched for the current frame, and before the current events are cleared. Causing some events to be lost. Thanks to [@andzdroid](https://github.com/andzdroid) for identifying and documenting the issue.\n- fixes a bug where temporary timers would crash. Thanks to [@holykol](https://github.com/holykol) for finding and fixing the issue.\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Features - Changelog","doc":"- imports `Ecspanse.Query` and `Ecspanse.Command` in all systems, so all the queries and commands are available without needing the respective module prefix.\n- imports `Ecspanse` in the setup module that `use Ecspanse` so the system scheduling functions are available without needing the module prefix.","ref":"changelog.html#features"},{"type":"extras","title":"v0.3.0 (2023-08-21) - Changelog","doc":"#","ref":"changelog.html#v0-3-0-2023-08-21"},{"type":"extras","title":"Features - Changelog","doc":"- adds a new query `Ecspanse.Query.list_tags/1` to list a component's tags.\n- adds a new query `Ecspanse.Query.list_components/1` to list all components of an entity.\n- adds a new command `Ecspanse.Command.clone_entity!/1` to clone an entity without its relationships.\n- adds a new command `Ecspanse.Command.deep_clone_entity!/1` to clone an entity with its descendants.","ref":"changelog.html#features"},{"type":"extras","title":"v0.2.1 (2023-08-20) - Changelog","doc":"#","ref":"changelog.html#v0-2-1-2023-08-20"},{"type":"extras","title":"Fixes - Changelog","doc":"- batch all events only by `batch_key` to avoid race conditions for different events processed by the same system.","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.2.0 (2023-08-18) - Changelog","doc":"#","ref":"changelog.html#v0-2-0-2023-08-18"},{"type":"extras","title":"Breaking - Changelog","doc":"- `use Ecspanse.Component.Timer` and `use Ecspanse.Event.Timer` are now deprecated.\n  Use `use Ecspanse.Template.Component.Timer` and `use Ecspanse.Template.Event.Timer` instead.\n\n#","ref":"changelog.html#breaking"},{"type":"extras","title":"Features - Changelog","doc":"- introducing `Ecspanse.Template.Component` and `Ecspanse.Template.Event` to simplify the creation of related components and events.\n- adds a new query `Ecspanse.Query.fetch_component/2` to fetch a system's component by a list of tags.","ref":"changelog.html#features"},{"type":"extras","title":"v0.1.2 (2023-08-14) - Changelog","doc":"#","ref":"changelog.html#v0-1-2-2023-08-14"},{"type":"extras","title":"Fixes - Changelog","doc":"- removes unneeded dependency `plug_crypto`\n- upgrades dependencies: `credo`, `ex_doc`, `jason`","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.1.1 (2023-08-12) - Changelog","doc":"#","ref":"changelog.html#v0-1-1-2023-08-12"},{"type":"extras","title":"Fixes - Changelog","doc":"- adds the missing project `:package`","ref":"changelog.html#fixes"}],"content_type":"text/markdown"}